/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import java.io.PrintStream;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import jline.console.completer.Completer;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.api.PetalsAdministrationFactory;
import org.ow2.petals.admin.api.RegistryAdministration;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.MissingServiceException;
import org.ow2.petals.admin.api.exception.RegistryAdministrationException;
import org.ow2.petals.admin.registry.Endpoint;
import org.ow2.petals.admin.registry.RegistryView;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.command.AbstractCommand;
import org.ow2.petals.cli.shell.completer.ContainerNameCompleter;

/**
 * This command registry-list a petals ESB container.
 * @author Sebastien Andre - EBM WebSourcing
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class ListRegistry extends AbstractCommand {

    private static final String DEFAULT_REGEX_OPTION_VALUE = ".*";

    /**
     * The container name regular expression option
     */
    private static final String CONTAINER_FILTER_SHORT_OPTION = "c";

    private static final String CONTAINER_FILTER_LONG_OPTION = "container";

    private static final String CONTAINER_FILTER_ARG_NAME = "container";

    private static final Option CONTAINER_FILTER_OPTION = OptionBuilder.hasArgs(1)
            .withArgName(CONTAINER_FILTER_ARG_NAME).withLongOpt(CONTAINER_FILTER_LONG_OPTION)
            .withDescription("Container name regular expression.").create(CONTAINER_FILTER_SHORT_OPTION);

    /**
     * The endpoint name regular expression option
     */
    private static final String ENDPOINT_NAME_SHORT_OPTION = "e";
    private static final String ENDPOINT_NAME_LONG_OPTION = "endpoint";
    private static final String ENDPOINT_NAME_ARG_NAME = "endpoint";
    private static final Option ENDPOINT_NAME_OPTION = OptionBuilder.
            hasArgs(1).withArgName(ENDPOINT_NAME_ARG_NAME)
            .withLongOpt(ENDPOINT_NAME_LONG_OPTION)
            .withDescription("Endpoint name regular expression.")
            .create(ENDPOINT_NAME_SHORT_OPTION);

    /**
     * The service name regular expression option
     */
    private static final String SERVICE_NAME_SHORT_OPTION = "s";
    private static final String SERVICE_NAME_LONG_OPTION = "service";
    private static final String SERVICE_NAME_ARG_NAME = "service";
    private static final Option SERVICE_NAME_OPTION = OptionBuilder.
            hasArgs(1).withArgName(SERVICE_NAME_ARG_NAME)
            .withLongOpt(SERVICE_NAME_LONG_OPTION)
            .withDescription("Service name regular expression.")
            .create(SERVICE_NAME_SHORT_OPTION);

    /**
     * The interface name regular expression option
     */
    private static final String INTERFACE_NAME_SHORT_OPTION = "i";
    private static final String INTERFACE_NAME_LONG_OPTION = "interface";
    private static final String INTERFACE_NAME_ARG_NAME = "interface";
    private static final Option INTERFACE_NAME_OPTION = OptionBuilder.
            hasArgs(1).withArgName(INTERFACE_NAME_ARG_NAME)
            .withLongOpt(INTERFACE_NAME_LONG_OPTION)
            .withDescription("Interface name regular expression.")
            .create(INTERFACE_NAME_SHORT_OPTION);

    /**
     * The flag selecting the part 'endpoints' for displaying
     */
    protected static final String OUTPUT_ENDPOINTS_LONG_OPTION = "output-endpoints";

    private static final Option OUTPUT_ENDPOINTS_OPTION = OptionBuilder.hasArg(false).isRequired(false)
            .withLongOpt(OUTPUT_ENDPOINTS_LONG_OPTION).withDescription("Output selector about endpoints.").create();

    /**
     * The flag selecting the part 'services' for displaying
     */
    protected static final String OUTPUT_SERVICES_LONG_OPTION = "output-services";

    private static final Option OUTPUT_SERVICES_OPTION = OptionBuilder.hasArg(false).isRequired(false)
            .withLongOpt(OUTPUT_SERVICES_LONG_OPTION).withDescription("Output selector about services.").create();

    /**
     * The flag selecting the part 'interfaces' for displaying
     */
    protected static final String OUTPUT_INTERFACES_LONG_OPTION = "output-interfaces";

    private static final Option OUTPUT_INTERFACES_OPTION = OptionBuilder.hasArg(false).isRequired(false)
            .withLongOpt(OUTPUT_INTERFACES_LONG_OPTION).withDescription("Output selector about interfaces.").create();

    private final PetalsAdministrationFactory paf;

    public ListRegistry() throws CommandException {
        super("registry-list");
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription("List the entries of the registry");

        try {
            this.paf = PetalsAdministrationFactory.newInstance();
        } catch (final DuplicatedServiceException e) {
            throw new CommandException(this, e);
        } catch (final MissingServiceException e) {
            throw new CommandException(this, e);
        }
    }

    @Override
    public void doExecute(final String[] args) throws CommandException {
        try {
            final CommandLineParser parser = new PosixParser();
            final CommandLine cmd = parser.parse(this.getOptions(), args);

            final String containerNameRegex = getRegexOption(cmd, CONTAINER_FILTER_SHORT_OPTION);
            final String endpointNameRegex = getRegexOption(cmd, ENDPOINT_NAME_SHORT_OPTION);
            final String serviceNameRegex = getRegexOption(cmd, SERVICE_NAME_SHORT_OPTION);
            final String interfaceNameRegex = getRegexOption(cmd, INTERFACE_NAME_SHORT_OPTION);

            final RegistryAdministration registryAdministration = this.paf.newRegistryAdministration();
            final RegistryView registryView = registryAdministration.getRegistryContent(containerNameRegex,
                    endpointNameRegex, serviceNameRegex, interfaceNameRegex);
            
            final boolean hasAnOutputFlag = cmd.hasOption(OUTPUT_ENDPOINTS_LONG_OPTION)
                    || cmd.hasOption(OUTPUT_SERVICES_LONG_OPTION) || cmd.hasOption(OUTPUT_INTERFACES_LONG_OPTION);
            final boolean isOutputEndpoints = cmd.hasOption(OUTPUT_ENDPOINTS_LONG_OPTION) || !hasAnOutputFlag;
            final boolean isOutputServices = cmd.hasOption(OUTPUT_SERVICES_LONG_OPTION) || !hasAnOutputFlag;
            final boolean isOutputInterfaces = cmd.hasOption(OUTPUT_INTERFACES_LONG_OPTION) || !hasAnOutputFlag;

            final StringBuilder registryContent = new StringBuilder();
            if (isOutputEndpoints) {
                if (isOutputServices || isOutputInterfaces) {
                    registryContent.append("Endpoints:\n");
                }
                final List<Endpoint> endpoints = registryView.getAllEndpoints();
                for (final Endpoint endpoint : endpoints) {
                    final String endpointInformation = String.format("%s: %s,%s,%s%n", endpoint.getEndpointName(),
                            endpoint.getContainerName(), endpoint.getComponentName(), endpoint.getEndpointType());
                    registryContent.append(endpointInformation);
                }
            }

            if (isOutputServices) {
                if (isOutputEndpoints || isOutputInterfaces) {
                    registryContent.append("Services:\n");
                }
                final Map<String, List<Endpoint>> endpointsByServiceName = registryView
                        .getListOfEndpointsByServiceName();
                for (final Entry<String, List<Endpoint>> entry : endpointsByServiceName.entrySet()) {
                    final String serviceName = entry.getKey();
                    registryContent.append(serviceName + ":\n");
                    int idx = 0;
                    for (final Endpoint endpoint : entry.getValue()) {
                        registryContent.append(endpoint.getEndpointName()
                                + (++idx == entry.getValue().size() ? "" : " "));
                    }
                    registryContent.append("\n");
                }
            }

            if (isOutputInterfaces) {
                if (isOutputEndpoints || isOutputServices) {
                    registryContent.append("Interfaces:\n");
                }
                final Map<String, List<Endpoint>> endpointsByInterfaceName = registryView
                        .getListOfEndpointsByInterfaceName();
                for (final Entry<String, List<Endpoint>> entry : endpointsByInterfaceName.entrySet()) {
                    registryContent.append(entry.getKey() + ":\n");
                    int idx = 0;
                    for (final Endpoint endpoint : entry.getValue()) {
                        registryContent.append(endpoint.getEndpointName()
                                + (++idx == entry.getValue().size() ? "" : " "));
                    }
                    registryContent.append("\n");
                }
            }

            final PrintStream out = this.getShell().getPrintStream();
            out.print(registryContent);
        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption());
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions());
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        } catch (final RegistryAdministrationException e) {
            throw new CommandException(this, e);
        }
    }

    private static final String getRegexOption(final CommandLine cmd, final String option) {
        final String endpointNameRegex;
        if (cmd.hasOption(option)) {
            endpointNameRegex = cmd.getOptionValue(option);
        } else {
            endpointNameRegex = DEFAULT_REGEX_OPTION_VALUE;
        }
        return endpointNameRegex;
    }

    @Override
    public Options createOptions() {
        final Options options = new Options();
        options.addOption(CONTAINER_FILTER_OPTION);
        options.addOption(ENDPOINT_NAME_OPTION);
        options.addOption(SERVICE_NAME_OPTION);
        options.addOption(INTERFACE_NAME_OPTION);
        options.addOption(OUTPUT_ENDPOINTS_OPTION);
        options.addOption(OUTPUT_SERVICES_OPTION);
        options.addOption(OUTPUT_INTERFACES_OPTION);
        return options;
    }

    @Override
    public Map<String, Completer> getOptionCompleters() {
        this.completers.put(CONTAINER_FILTER_SHORT_OPTION, new ContainerNameCompleter(this.paf));
        return this.completers;
    }
}
