/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import jline.console.completer.Completer;
import jline.console.completer.FileNameCompleter;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.api.ContainerAdministration;
import org.ow2.petals.admin.api.PetalsAdministrationFactory;
import org.ow2.petals.admin.api.exception.ContainerAdministrationException;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.MissingServiceException;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.command.exception.CommandMissingYesFlagException;
import org.ow2.petals.cli.api.command.exception.CommandUnparsableArgumentException;
import org.ow2.petals.cli.api.shell.exception.ShellException;
import org.ow2.petals.cli.command.AbstractCommand;
import org.ow2.petals.jmx.api.api.JMXClient;

/**
 * The Petals CLI command 'move-container'
 * @author Christophe DENEUX - Linagora
 */
public class MoveContainer extends AbstractCommand {

    /**
     * The option defining the target sub-domain name
     */
    public static final String TARGET_SUBDOMAIN_LONG_OPTION = "target-subdomain";

    protected static final Option TARGET_SUBDOMAIN_OPTION = OptionBuilder.isRequired(false).hasArgs(1)
            .withArgName("subdomain").withLongOpt(TARGET_SUBDOMAIN_LONG_OPTION)
            .withDescription("The target sub-domain name, the one that will be joined.")
            .create();

    /**
     * The option defining the target container name
     */
    public static final String TARGET_NAME_LONG_OPTION = "target-name";

    protected static final Option TARGET_NAME_OPTION = OptionBuilder
            .isRequired(false)
            .hasArgs(1)
            .withArgName("container").withLongOpt(TARGET_NAME_LONG_OPTION)
            .withDescription("The name of the container of the target sub-domain, used as entry point in the target sub-domain.")
            .create();

    /**
     * The option defining the target host name
     */
    public static final String TARGET_HOST_LONG_OPTION = "target-host";

    protected static final Option TARGET_HOST_OPTION = OptionBuilder.isRequired(false).hasArgs(1)
            .withArgName("hostname").withLongOpt(TARGET_HOST_LONG_OPTION)
            .withDescription("The host name of the container of the target sub-domain. Default value: localhost")
            .create();

    /**
     * The option defining the jmx port of the target container
     */
    public static final String TARGET_PORT_LONG_OPTION = "target-port";

    protected static final Option TARGET_PORT_OPTION = OptionBuilder
            .isRequired(false)
            .hasArgs(1)
            .withArgName("jmx-port")
            .withLongOpt(TARGET_PORT_LONG_OPTION)
            .withDescription("The JMX port of the container of the target sub-domain. Default value: " + JMXClient.DEFAULT_PORT)
            .create();

    /**
     * The option defining the jmx username of the target container
     */
    public static final String TARGET_USER_LONG_OPTION = "target-user";

    protected static final Option TARGET_USER_OPTION = OptionBuilder.isRequired(false).hasArgs(1)
            .withArgName("jmx-user").withLongOpt(TARGET_USER_LONG_OPTION)
            .withDescription("The JMX username of the container of the target sub-domain. Default value: petals")
            .create();

    /**
     * The option defining the jmx password of the target container
     */
    public static final String TARGET_PWD_LONG_OPTION = "target-pwd";

    protected static final Option TARGET_PWD_OPTION = OptionBuilder.isRequired(false).hasArgs(1)
            .withArgName("jmx-password").withLongOpt(TARGET_PWD_LONG_OPTION)
            .withDescription("The JMX password of the container of the target sub-domain. Default value: petals")
            .create();

    public static final String ATTACH_CONFIRMATION_MSG = "Are you sure you want to move the current container? (y/n) ";

    public static final String DETACH_CONFIRMATION_MSG = "Are you sure you want to detach the current container? (y/n) ";

    /**
     * The option defining the pass-phrase of the target container
     */
    public static final String TARGET_PASSPHRASE_LONG_OPTION = "target-pass-phrase";

    protected static final Option PASSPHRASE_OPTION = OptionBuilder.isRequired(false).hasArgs(1)
            .withArgName("target-pass-phrase").withLongOpt(TARGET_PASSPHRASE_LONG_OPTION)
            .withDescription("The security pass-phrase to access sensible information.").create();

    /**
     * The confirmation skip option
     */
    public static final String YESFLAG_SHORT_OPTION = "y";

    private static final Option YESFLAG_OPTION = OptionBuilder.hasArg(false)
            .withDescription("A flag to skip confirmation.").create(YESFLAG_SHORT_OPTION);

    public MoveContainer() {
        super("move-container");
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription("Move a container from a sub-domain to another one.");
        this.setOptionsDescription(CommandUtil.formatCommandOptionsDescription(this));
        final FileNameCompleter c = new FileNameCompleter();
        this.completers.put(TARGET_SUBDOMAIN_LONG_OPTION, c);
        this.completers.put(TARGET_NAME_LONG_OPTION, c);
        this.completers.put(TARGET_HOST_LONG_OPTION, c);
        this.completers.put(TARGET_PORT_LONG_OPTION, c);
        this.completers.put(TARGET_USER_LONG_OPTION, c);
        this.completers.put(TARGET_PWD_LONG_OPTION, c);
        this.completers.put(TARGET_PASSPHRASE_LONG_OPTION, c);
    }

    @Override
    public void doExecute(final String[] args) throws CommandException {
        

        final CommandLineParser parser = new PosixParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);
            
            if (cli.hasOption(TARGET_NAME_LONG_OPTION) && cli.hasOption(TARGET_SUBDOMAIN_LONG_OPTION)) {

                // Attaching or moving a container to a sub-domain

                final String targetSubdomainName = cli.getOptionValue(TARGET_SUBDOMAIN_LONG_OPTION);
                final String targetContainerName = cli.getOptionValue(TARGET_NAME_LONG_OPTION);
                final String targetContainerHost;
                if (cli.hasOption(TARGET_HOST_LONG_OPTION)) {
                    targetContainerHost = cli.getOptionValue(TARGET_HOST_LONG_OPTION);
                } else {
                    targetContainerHost = "localhost";
                }
                final int targetContainerPort;
                if (cli.hasOption(TARGET_PORT_LONG_OPTION)) {
                    final String targetContainerPortStr = cli.getOptionValue(TARGET_PORT_LONG_OPTION);
                    try {
                        targetContainerPort = Integer.parseInt(targetContainerPortStr);
                    } catch (final NumberFormatException e) {
                        throw new CommandUnparsableArgumentException(this, TARGET_PORT_OPTION, targetContainerPortStr);
                    }
                } else {
                    targetContainerPort = JMXClient.DEFAULT_PORT;
                }
                final String targetContainerUser;
                if (cli.hasOption(TARGET_USER_LONG_OPTION)) {
                    targetContainerUser = cli.getOptionValue(TARGET_USER_LONG_OPTION);
                } else {
                    targetContainerUser = "petals";
                }
                final String targetContainerPwd;
                if (cli.hasOption(TARGET_PWD_LONG_OPTION)) {
                    targetContainerPwd = cli.getOptionValue(TARGET_PWD_LONG_OPTION);
                } else {
                    targetContainerPwd = "petals";
                }
                final String securityPassPhrase;
                if (cli.hasOption(TARGET_PASSPHRASE_LONG_OPTION)) {
                    securityPassPhrase = cli.getOptionValue(TARGET_PASSPHRASE_LONG_OPTION);
                } else {
                    final List<Option> missingOptions = new ArrayList<Option>();
                    missingOptions.add(PASSPHRASE_OPTION);
                    throw new CommandMissingOptionsException(this, missingOptions);
                }

                if (!cli.hasOption(YESFLAG_SHORT_OPTION)) {
                    if (this.getShell().confirms(ATTACH_CONFIRMATION_MSG)) {
                        this.moveContainer(targetSubdomainName, targetContainerName, targetContainerHost,
                                targetContainerPort, targetContainerUser, targetContainerPwd, securityPassPhrase);
                    } else if (!this.getShell().isInteractive()) {
                        throw new CommandMissingYesFlagException(this, YESFLAG_OPTION);
                    }
                } else {
                    this.moveContainer(targetSubdomainName, targetContainerName, targetContainerHost,
                            targetContainerPort, targetContainerUser, targetContainerPwd, securityPassPhrase);
                }
            } else if (!cli.hasOption(TARGET_NAME_LONG_OPTION) && !cli.hasOption(TARGET_SUBDOMAIN_LONG_OPTION)
                    && !cli.hasOption(TARGET_HOST_LONG_OPTION) && !cli.hasOption(TARGET_PORT_LONG_OPTION)
                    && !cli.hasOption(TARGET_USER_LONG_OPTION) && !cli.hasOption(TARGET_PWD_LONG_OPTION)
                    && !cli.hasOption(TARGET_PASSPHRASE_LONG_OPTION)) {

                // Detaching a container from a sub-domain

                if (!cli.hasOption(YESFLAG_SHORT_OPTION)) {
                    if (this.getShell().confirms(DETACH_CONFIRMATION_MSG)) {
                        this.detachContainer();
                    } else if (!this.getShell().isInteractive()) {
                        throw new CommandMissingYesFlagException(this, YESFLAG_OPTION);
                    }
                } else {
                    this.detachContainer();
                }

            } else {
                throw new CommandBadArgumentNumberException(this);
            }

        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption());
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions());
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        } catch (final ContainerAdministrationException e) {
            throw new CommandException(this, e);
        } catch (final ShellException e) {
            throw new CommandException(this, e);
        }
    }

    /**
     * Move the container on which we are connected
     * 
     * @param targetSubdomainName
     *            The target sub-domain name, the one that will be joined
     * @param targetContainerName
     *            The name of the container of the target sub-domain, used as entry point in the target sub-domain
     * @param targetContainerHost
     *            The host name of the container of the target sub-domain
     * @param targetContainerPort
     *            The JMX port of the container of the target sub-domain
     * @param targetContainerUser
     *            The JMX username of the container of the target sub-domain
     * @param targetContainerPwd
     *            The JMX password of the container of the target sub-domain
     * @param securityPassPhrase
     *            The security pass-phrase of the target container to access sensible information.
     */
    private void moveContainer(final String targetSubdomainName, final String targetContainerName,
            final String targetContainerHost, final int targetContainerPort, final String targetContainerUser,
            final String targetContainerPwd, final String securityPassPhrase) throws ContainerAdministrationException {

        try {
            final ContainerAdministration containerAdministration = PetalsAdministrationFactory.newInstance()
                    .newContainerAdministration();
            containerAdministration.attachContainer(targetSubdomainName, targetContainerName, targetContainerHost,
                    targetContainerPort, targetContainerUser, targetContainerPwd, securityPassPhrase);

            this.getShell().setDefaultPrompt();
        } catch (final DuplicatedServiceException e) {
            throw new ContainerAdministrationException(e);
        } catch (final MissingServiceException e) {
            throw new ContainerAdministrationException(e);
        }
    }

    /**
     * Detach the container on which we are connected
     */
    private void detachContainer() throws ContainerAdministrationException {

        try {
            final ContainerAdministration containerAdministration = PetalsAdministrationFactory.newInstance()
                    .newContainerAdministration();
            containerAdministration.detachContainer();

            this.getShell().setDefaultPrompt();
        } catch (final DuplicatedServiceException e) {
            throw new ContainerAdministrationException(e);
        } catch (final MissingServiceException e) {
            throw new ContainerAdministrationException(e);
        }
    }

    @Override
    public Options createOptions() {
        final Options options = new Options();
        options.addOption(TARGET_SUBDOMAIN_OPTION);
        options.addOption(TARGET_NAME_OPTION);
        options.addOption(TARGET_HOST_OPTION);
        options.addOption(TARGET_PORT_OPTION);
        options.addOption(TARGET_USER_OPTION);
        options.addOption(TARGET_PWD_OPTION);
        options.addOption(PASSPHRASE_OPTION);
        options.addOption(YESFLAG_OPTION);
        return options;
    }

    @Override
    public Map<String, Completer> getOptionCompleters() {
        return this.completers;
    }
}
