/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import java.io.PrintStream;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;

import jline.console.completer.Completer;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.api.ContainerAdministration;
import org.ow2.petals.admin.api.PetalsAdministrationFactory;
import org.ow2.petals.admin.api.exception.ContainerAdministrationException;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.MissingServiceException;
import org.ow2.petals.admin.topology.Container;
import org.ow2.petals.admin.topology.Container.PortType;
import org.ow2.petals.admin.topology.Domain;
import org.ow2.petals.admin.topology.Subdomain;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.connection.ConnectionParameters;
import org.ow2.petals.cli.command.AbstractCommand;
import org.ow2.petals.cli.connection.AuthenticatedConnectionParametersImpl;
import org.ow2.petals.cli.shell.completer.ContainerNameCompleter;

/**
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class TopologyList extends AbstractCommand {

    private static final String DEFAULT_REGEX_OPTION_VALUE = ".*";

    public final static String PASSPHRASE_SHORT_OPTION = "p";

    public final static String PASSPHRASE_LONG_OPTION = "pass-phrase";

    protected static final Option PASSPHRASE_OPTION = OptionBuilder.isRequired(false).hasArgs(1)
            .withArgName("pass-phrase").withLongOpt(PASSPHRASE_LONG_OPTION)
            .withDescription("The security pass-phrase to get critical information.")
            .create(PASSPHRASE_SHORT_OPTION);

    public final static String CONTAINER_FILTER_SHORT_OPTION = "c";

    public final static String CONTAINER_FILTER_LONG_OPTION = "container";

    protected static final Option CONTAINER_FILTER_OPTION = OptionBuilder.isRequired(false).hasOptionalArg()
            .withArgName("container-name-regexp").withLongOpt(CONTAINER_FILTER_LONG_OPTION)
            .withDescription("Regexp to use as container name filter.").create(CONTAINER_FILTER_SHORT_OPTION);

    private final PetalsAdministrationFactory paf;

    public TopologyList() throws CommandException {
        super("topology-list");
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription("Get information about the topology this node is part of.");
        this.setOptionsDescription(CommandUtil.formatCommandOptionsDescription(this));
        try {
            this.paf = PetalsAdministrationFactory.newInstance();
        } catch (final DuplicatedServiceException e) {
            throw new CommandException(this, e);
        } catch (final MissingServiceException e) {
            throw new CommandException(this, e);
        }
    }

    @Override
    public void doExecute(final String[] args) throws CommandException {
        

        final CommandLineParser parser = new PosixParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);
            
            final String passPhrase;
            if (cli.hasOption(PASSPHRASE_SHORT_OPTION)) {
                // The pass phrase is provided as argument option
                passPhrase = cli.getOptionValue(PASSPHRASE_SHORT_OPTION);
            } else if (!cli.getArgList().isEmpty()) {
                throw new CommandBadArgumentNumberException(this);
            } else {
                // We check that the pass phrase is set at alias level
                final ConnectionParameters connectionParameters = this.getShell().getConnectionParameters();

                if (connectionParameters == null) {
                    passPhrase = null;
                } else if (connectionParameters instanceof AuthenticatedConnectionParametersImpl) {
                    passPhrase = ((AuthenticatedConnectionParametersImpl) connectionParameters).getPassPhrase();
                } else {
                    passPhrase = null;
                }
            }
            
            final ContainerAdministration containerAdministration = this.paf.newContainerAdministration();
            final Properties currentServerProps = containerAdministration.getServerProperties();
            // TODO: The property 'petals.container.name' must be defined in an API usable by Petals CLI
            final String defaultContainerRegexp = currentServerProps == null ? DEFAULT_REGEX_OPTION_VALUE
                    : currentServerProps.getProperty("petals.container.name");

            final String containerNameRegex = getRegexOption(cli, CONTAINER_FILTER_SHORT_OPTION, defaultContainerRegexp);

            final Domain domain = containerAdministration.getTopology(containerNameRegex, passPhrase);
            if (domain == null) {
                throw new CommandException(this, "No Petals domain found");
            }
            final StringBuilder topologyInformation = new StringBuilder(String.format("Domain: %s%n",
                    domain.getName()));

            for (final Subdomain subdomain : domain.getSubdomains()) {
                topologyInformation.append(String.format("* Subdomain: %s%n", subdomain.getName()));

                for (Container container : subdomain.getContainers()) {
                    final String containerInformation = String.format("** Container: %s (%s)%n*** Host: %s%n",
                            container.getContainerName(), container.getState(), container.getHost());
                    topologyInformation.append(containerInformation);

                    final Map<PortType, Integer> ports = container.getPorts();
                    for (final Entry<PortType, Integer> entry : ports.entrySet()) {
                        topologyInformation.append(String.format("*** %s: %d%n", entry.getKey(), entry.getValue()));
                    }

                    final String jmxUser = container.getJmxUsername();
                    topologyInformation.append(String.format("*** JMX username: %s%n",
                            (jmxUser == null || jmxUser.isEmpty()) ? "*********" : jmxUser));
                    final String jmxPassword = container.getJmxPassword();
                    topologyInformation.append(String.format("*** JMX password: %s%n",
                            (jmxPassword == null || jmxPassword.isEmpty()) ? "*********"
                                    : jmxPassword));

                }
            }

            final PrintStream out = this.getShell().getPrintStream();
            out.println(topologyInformation);

        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption());
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions());
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        } catch (final ContainerAdministrationException e) {
            throw new CommandException(this, e);
        }
    }

    @Override
    public Options createOptions() {
        final Options options = new Options();
        options.addOption(CONTAINER_FILTER_OPTION);
        options.addOption(PASSPHRASE_OPTION);
        return options;
    }

    @Override
    public Map<String, Completer> getOptionCompleters() {
        this.completers.put(CONTAINER_FILTER_SHORT_OPTION, new ContainerNameCompleter(this.paf));
        return this.completers;
    }

    /**
     * 
     * @param cmd
     * @param option
     * @param defaultRegexp
     *            A default regexp to use if no option value is defined
     * @return
     */
    private static final String getRegexOption(final CommandLine cmd, final String option, final String defaultRegexp) {
        final String regexp;
        if (cmd.hasOption(option)) {
            final String value = cmd.getOptionValue(option);
            if (value == null) {
                regexp = defaultRegexp;
            } else {
                regexp = cmd.getOptionValue(option);
            }
        } else {
            regexp = DEFAULT_REGEX_OPTION_VALUE;
        }
        return regexp;
    }
}
