/**
 * Copyright (c) 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;
import java.io.Writer;
import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Map;

import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.ow2.petals.admin.ContainerAdministrationMock;
import org.ow2.petals.admin.PetalsAdministrationFactoryMock;
import org.ow2.petals.admin.api.ContainerAdministration;
import org.ow2.petals.admin.api.PetalsAdministrationFactory;
import org.ow2.petals.admin.api.exception.ContainerAdministrationException;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.MissingServiceException;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.connection.ConnectionParameters;
import org.ow2.petals.cli.api.connection.exception.HostMissingException;
import org.ow2.petals.cli.api.connection.exception.InvalidPortException;
import org.ow2.petals.cli.api.connection.exception.PasswordMissingException;
import org.ow2.petals.cli.api.connection.exception.UsernameMissingException;
import org.ow2.petals.cli.api.pref.PreferenceFileException;
import org.ow2.petals.cli.api.pref.Preferences;
import org.ow2.petals.cli.api.pref.exception.MissingDefaultPreferenceFileException;
import org.ow2.petals.cli.api.pref.exception.PreferenceFileNotFoundException;
import org.ow2.petals.cli.api.shell.exception.DuplicatedCommandException;
import org.ow2.petals.cli.base.junit.shell.PetalsInteractiveCliUtils;
import org.ow2.petals.cli.command.Help;
import org.ow2.petals.cli.connection.AuthenticatedConnectionParametersImpl;
import org.ow2.petals.cli.shell.ShellFactory;
import org.ow2.petals.cli.shell.command.AbstractArtifactCommandTest;
import org.ow2.petals.cli.shell.command.Connect;
import org.ow2.petals.cli.shell.command.Version;

/**
 * Unit test of {@link Main}.
 * @author Christophe DENEUX - Linagora
 */
public class MainTest {

    /**
     * Assertion exception used by threads
     */
    private volatile AssertionError assertion;

    @Rule
    public final TemporaryFolder tempFolder = new TemporaryFolder();

    @Before
    public void beforeTest() {
        System.setProperty(PetalsAdministrationFactory.PROPERTY_NAME,
                PetalsAdministrationFactoryMock.class.getName());
    }

    @After
    public void afterTest() {
        System.setProperty(PetalsAdministrationFactory.PROPERTY_NAME, "");
    }

    /**
     * <p>
     * Check the main entry point against a connection establishment on a unresolvable hostname in command line mode,
     * ie. a command line containing:
     * <ul>
     * <li>the command mode without connection arguments,</li>
     * <li>the operand flag,</li>
     * <li>a command requiring a connection,</li>
     * <li>a connection to a hostname unresolvable.</li>
     * </ul>
     * as following:<br/>
     * <code>> ./petals-cli.sh -h unresolvable-host -n 7700 -p petals -u petals -c -- list</code>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error about the unresovable hostname,</li>
     * <li>return code 1.</li>
     * </ul>
     * </p>
     */
    @Test
    public final void testRun_Command_ConnectionToUnresolvableHostname()
            throws DuplicatedServiceException, MissingServiceException,
            ContainerAdministrationException {

        final PrintStream standardConsoleOutput = System.out;
        final PrintStream standardConsoleError = System.err;

        try {
            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));
            final ByteArrayOutputStream outBAIS = new ByteArrayOutputStream();
            System.setOut(new PrintStream(outBAIS));

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();

            final Command helpCommand = new Help();
            final String unresolvableHostname = "unresolvable-host";
            final int exitCode = main.run(new String[] {
                    "-" + Connect.HOST_SHORT_OPTION, unresolvableHostname,
                    "-" + Connect.PORT_SHORT_OPTION, "7700",
                    "-" + Connect.USER_SHORT_OPTION, "petals",
                    "-" + Connect.PWD_SHORT_OPTION, "petals",
                    "-" + ShellFactory.COMMAND_SHORT_OPTION,
                    "--",
                    helpCommand.getName() }, null, null);

            final String error = errBAIS.toString();
            final String output = outBAIS.toString();
            assertEquals("invalid exit code", 1, exitCode);
            assertFalse("Usage is not print", output.isEmpty());
            assertFalse("Error message is empty", error.isEmpty());
            assertTrue("Unresolvable hostname is missing on the error message",
                    error.contains(unresolvableHostname));
        } finally {
            System.setOut(standardConsoleOutput);
            System.setErr(standardConsoleError);
        }
    }

    /**
     * Test the correct disconnection when:
     * <ul>
     * <li>a connection is established using the yes flag,</li>
     * <li>a command requiring the connection is executed (ex: 'version' that
     * requires a connection in the implementation of the Petals Admin API
     * mock),</li>
     * <li>the command 'disconnect' or 'exit' is not executed before to end the
     * Petals CLI script,</li>
     * <li>using the inlined mode.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>No error occurs,</li>
     * <li>The connection is established on 'connect' from preferences,</li>
     * <li>The connection is correctly disconnected.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testRun_Inlined_Disconnection_00() throws ContainerAdministrationException,
            DuplicatedServiceException, MissingServiceException {

        final PrintStream standardConsoleOutput = System.out;
        final PrintStream standardConsoleError = System.err;
        final InputStream standardConsoleInput = System.in;

        try {
            final ByteArrayOutputStream outBAIS = new ByteArrayOutputStream();
            System.setOut(new PrintStream(outBAIS));
            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();

            final StringBuilder commandFlow = new StringBuilder("connect -y")
                    .append(System.getProperty("line.separator")).append("version")
                    .append(System.getProperty("line.separator"));
            final ByteArrayInputStream inBAIS = new ByteArrayInputStream(commandFlow.toString()
                    .getBytes());
            System.setIn(inBAIS);

            final int exitCode = main.run(new String[] { "-h", "localhost", "-n", "7700", "-u", "petals", "-p",
                    "petals", "-" }, this.createBasicPreferences("localhost", 7700, "petals", "petals", null), null);

            final String error = errBAIS.toString();
            final String output = outBAIS.toString();
            assertTrue("Connection not established",
                    output.contains(ContainerAdministrationMock.SYSTEM_INFO));
            assertTrue("An error occurs", error.isEmpty());
            assertEquals("Invalid exit code", 0, exitCode);
            final ContainerAdministration containerAdministration = PetalsAdministrationFactory
                    .newInstance().newContainerAdministration();
            assertFalse("Connection not disconnected", containerAdministration.isConnected());
        } finally {
            System.setOut(standardConsoleOutput);
            System.setErr(standardConsoleError);
            System.setIn(standardConsoleInput);
        }

    }

    /**
     * Test the correct disconnection when:
     * <ul>
     * <li>a connection is established using the yes flag,</li>
     * <li>a command requiring the connection is executed (ex: 'version' that
     * requires a connection in the implementation of the Petals Admin API
     * mock),</li>
     * <li>the command 'disconnect' or 'exit' is not executed before to end the
     * Petals CLI script,</li>
     * <li>using a non-interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>No error occurs,</li>
     * <li>The connection is established on 'connect',</li>
     * <li>The connection is correctly disconnected.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testRun_File_Disconnection_00() throws CommandException,
            DuplicatedCommandException,
            IllegalArgumentException, IOException, DuplicatedServiceException,
            MissingServiceException, ContainerAdministrationException {

        final PrintStream standardConsoleOut = System.out;
        final PrintStream standardConsoleError = System.err;

        final Command commandConnect = new Connect();
        final Command commandVersion = new Version();

        // Create the Petals CLI script
        final File petalsCliScript = this.tempFolder.newFile("petals-cli.cmd");
        final OutputStream osScript = new FileOutputStream(petalsCliScript);
        final Writer pwScript = new OutputStreamWriter(osScript);
        try {
            pwScript.write(commandConnect.getName());
            pwScript.write(" -y\n");
            pwScript.write(commandVersion.getName());
            pwScript.write('\n');
        } finally {
            pwScript.close();
            osScript.close();
        }

        try {
            final ByteArrayOutputStream outBAIS = new ByteArrayOutputStream();
            System.setOut(new PrintStream(outBAIS));
            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();

            final int exitCode = main.run(
                    new String[] { "-" + ShellFactory.FILE_LONG_OPTION, petalsCliScript.getAbsolutePath() },
                    this.createBasicPreferences("localhost", 7700, "petals", "petals", null), null);

            final String error = errBAIS.toString();
            final String output = outBAIS.toString();

            assertTrue("Connection not established",
                    output.contains(ContainerAdministrationMock.SYSTEM_INFO));
            assertTrue("An error occurs", error.isEmpty());
            assertEquals("Invalid exit code", 0, exitCode);
            final ContainerAdministration containerAdministration = PetalsAdministrationFactory
                    .newInstance().newContainerAdministration();
            assertFalse("Connection not disconnected", containerAdministration.isConnected());

        } finally {
            System.setOut(standardConsoleOut);
            System.setErr(standardConsoleError);
        }
    }

    /**
     * Test the correct disconnection when:
     * <ul>
     * <li>using the command line shell,</li>
     * <li>a automatic connection is established,</li>
     * <li>a command requiring the connection is executed (ex: 'version' that requires a connection in the
     * implementation of the Petals Admin API mock).</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>No error occurs, connection parameters are read from the preference files</li>
     * <li>The connection is established before to execute 'version',</li>
     * <li>The connection is correctly disconnected when command line ends.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testRun_Command_Disconnection_00() throws CommandException,
            DuplicatedCommandException,
            IllegalArgumentException, IOException, DuplicatedServiceException,
            MissingServiceException, ContainerAdministrationException {

        final PrintStream standardConsoleOut = System.out;
        final PrintStream standardConsoleError = System.err;

        final Command commandVersion = new Version();

        try {
            final ByteArrayOutputStream outBAIS = new ByteArrayOutputStream();
            System.setOut(new PrintStream(outBAIS));
            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();

            final int exitCode = main.run(
                    new String[] { "-" + ShellFactory.COMMAND_SHORT_OPTION, "--", commandVersion.getName() },
                    this.createBasicPreferences("localhost", 7700, "petals", "petals", null),
                    null);

            final String error = errBAIS.toString();
            final String output = outBAIS.toString();

            assertTrue("Connection not established",
                    output.contains(ContainerAdministrationMock.SYSTEM_INFO));
            assertTrue("An error occurs", error.isEmpty());
            assertEquals("Invalid exit code", 0, exitCode);
            final ContainerAdministration containerAdministration = PetalsAdministrationFactory
                    .newInstance().newContainerAdministration();
            assertFalse("Connection not disconnected", containerAdministration.isConnected());

        } finally {
            System.setOut(standardConsoleOut);
            System.setErr(standardConsoleError);
        }
    }

    /**
     * Create a basic preference configuration
     * 
     * @param host
     * @param port
     * @param username
     * @param password
     * @param passPhrase
     * @return A basic preference configuration
     * @deprecated Must be mutualized with {@link AbstractArtifactCommandTest#createBasicPreferences}
     */
    @Deprecated
    private Preferences createBasicPreferences(final String host, final int port, final String username,
            final String password, final String passPhrase) {

        try {
            final ConnectionParameters connectionParameters = new AuthenticatedConnectionParametersImpl(host, port,
                    username, password, passPhrase);
            return new Preferences() {

                @Override
                public void intializePreferenceParameters() throws MissingDefaultPreferenceFileException,
                        PreferenceFileNotFoundException, PreferenceFileException {
                    // NOP
                }

                @Override
                public Map<String, ConnectionParameters> getPreferenceConnectionParameters() {
                    final Map<String, ConnectionParameters> connectionParametersMap = new HashMap<String, ConnectionParameters>();
                    connectionParametersMap.put("default", connectionParameters);
                    return connectionParametersMap;
                }

                @Override
                public ConnectionParameters getDefaultConnectionParameters() {
                    return connectionParameters;
                }
            };
        } catch (final UnknownHostException e) {
            // NOP: This exception has not to occur
            return null;
        } catch (final HostMissingException e) {
            // NOP: This exception has not to occur
            return null;
        } catch (final InvalidPortException e) {
            // NOP: This exception has not to occur
            return null;
        } catch (final UsernameMissingException e) {
            // NOP: This exception has not to occur
            return null;
        } catch (final PasswordMissingException e) {
            // NOP: This exception has not to occur
            return null;
        }
    }

    /**
     * Test the correct disconnection when:
     * <ul>
     * <li>a connection is established without using the yes flag but confirming
     * connection values,</li>
     * <li>a command requiring the connection is executed (ex: 'version' that
     * requires a connection in the implementation of the Petals Admin API
     * mock),</li>
     * <li>the command 'disconnect' or 'exit' is not executed before to end the
     * Petals CLI script,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>No error occurs,</li>
     * <li>The connection is established on 'connect' from preferences,</li>
     * <li>The connection is correctly disconnected.</li>
     * </ul>
     * </p>
     */
    @Test()
    public void testRun_Interactive_Disconnection_00() throws CommandException,
            DuplicatedServiceException,
            MissingServiceException, ContainerAdministrationException, IOException,
            InterruptedException {

        final PrintStream standardConsoleOutput = System.out;
        final PrintStream standardConsoleError = System.err;
        final InputStream standardConsoleInput = System.in;

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));
            final ByteArrayOutputStream outBAIS = new ByteArrayOutputStream();
            System.setOut(new PrintStream(outBAIS));
            final PipedOutputStream pos = new PipedOutputStream();
            System.setIn(new PipedInputStream(pos));

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();
            final String hostname = "localhost";

            this.assertion = null;
            final ContainerAdministration containerAdministration = PetalsAdministrationFactory
                    .newInstance().newContainerAdministration();
            final Runnable writer = new Runnable() {

                @Override
                public void run() {
                    try {
                        try {
                            // Wait that Petals CLI starts
                            Thread.sleep(1000);

                            final String output1 = outBAIS.toString();
                            final String error1 = errBAIS.toString();
                            assertFalse("Prompt is empty", output1.isEmpty());
                            assertTrue("An error occurs", error1.isEmpty());
                            assertTrue("Invalid prompt", output1.contains("petals-cli>"));
                            assertFalse("Invalid prompt", output1.contains(hostname + ":7700"));

                            // Connect
                            pos.write("connect\n".getBytes());
                            // Wait the execution of the command
                            Thread.sleep(1000);
                            final String output2 = outBAIS.toString().substring(output1.length());
                            final String error2 = errBAIS.toString().substring(error1.length());
                            assertFalse("Prompt is empty", output2.isEmpty());
                            assertTrue("An error occurs", error2.isEmpty());
                            assertTrue(
                                    "Confirmation message is missing",
                                    output2.contains("Would you like to connect to petals:*****@"
                                            + hostname + ":7700? (y/n)"));

                            // Confirmation
                            pos.write("y\n".getBytes());
                            // Wait the execution of the confirmation
                            Thread.sleep(1000);
                            final String output3 = outBAIS.toString().substring(
                                    output1.length() + output2.length());
                            final String error3 = errBAIS.toString().substring(
                                    error1.length() + error2.length());
                            assertFalse("Prompt is empty", output3.isEmpty());
                            assertTrue("An error occurs", error3.isEmpty());
                            assertTrue("Petals CLI is not connected",
                                    output3.contains("petals-cli@" + hostname + ":7700>"));
                            assertTrue("Connection not established",
                                    containerAdministration.isConnected());

                            // Enter the command
                            pos.write("version\n".getBytes());
                            // Wait the execution of the command
                            Thread.sleep(1000);
                            final String output4 = outBAIS.toString().substring(
                                    output1.length() + output2.length() + output3.length());
                            final String error4 = errBAIS.toString().substring(
                                    error1.length() + error2.length() + error3.length());
                            assertTrue("An error occurs", error4.isEmpty());
                            assertTrue("Unexpected version information",
                                    output4.contains(ContainerAdministrationMock.SYSTEM_INFO));

                        } catch (final Exception e) {
                            fail("An exception occurs: " + e.getMessage());
                        }
                    } catch (final AssertionError e) {
                        MainTest.this.assertion = e;
                    }
                }
            };

            final Runnable petalsCli = new Runnable() {

                @Override
                public void run() {

                    try {
                        final int exitCode = main
                                .run(new String[] { "-h", hostname, "-n", "7700", "-u", "petals", "-p", "petals", "-C" },
                                        MainTest.this.createBasicPreferences("localhost", 7700, "petals", "petals",
                                                null), null);

                        try {
                            assertEquals("invalid exit code", 0, exitCode);
                            assertFalse("Connection not disconnected",
                                    containerAdministration.isConnected());
                        } catch (final Exception e) {
                            fail("An exception occurs: " + e.getMessage());
                        }

                    } catch (final AssertionError e) {
                        MainTest.this.assertion = e;
                    }

                }
            };

            final Thread petalsCliThread = new Thread(petalsCli, "petalsCliThread");
            final Thread writerThread = new Thread(writer, "writerThread");
            petalsCliThread.start();
            writerThread.start();
            writerThread.join();
            pos.close();
            petalsCliThread.join();

            if (this.assertion != null) {
                throw this.assertion;
            }
        } finally {
            System.setOut(standardConsoleOutput);
            System.setErr(standardConsoleError);
            System.setIn(standardConsoleInput);

            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * <p>
     * Check the main entry point against an command line containing to check the host name in prompt:
     * <ul>
     * <li>the flag debug is disable,</li>
     * <li>connection defines on the command line with a host defined by its <b>IP address</b>,</li>
     * <li>the interactive mode,</li>
     * <li>the connection is done with 'connect' without args, and connection is confirmed,</li>
     * <li>no other command is executed.</li>
     * </ul>
     * as following:<br/>
     * <code>
     * > ./petals-cli.sh -h 127.0.0.1 -n 7700 -u petals -p petals -C<br/>
     * petals-cli> connect
     * petals-cli@localhost:7700>
     * </code>
     * </p>
     * <p>
     * Note: the connection <b>is not a real</b> connection.
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>the prompt displays connection information with an explicit hostname,</li>
     * <li>no error,</li>
     * <li>no error stacktrace is printed,</li>
     * <li>no usage is printed.</li>
     * </ul>
     * </p>
     */
    @Test
    public final void testRun_Interactive_ExplicitHostname_00() throws IOException, InterruptedException {

        final PrintStream standardConsoleOutput = System.out;
        final PrintStream standardConsoleError = System.err;
        final InputStream standardConsoleInput = System.in;

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            // Go on...
            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));
            final ByteArrayOutputStream outBAIS = new ByteArrayOutputStream();
            System.setOut(new PrintStream(outBAIS));
            final PipedOutputStream pos = new PipedOutputStream();
            System.setIn(new PipedInputStream(pos));

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();
            final String hostname = "127.0.0.1";
            final String expectedHostname = "localhost";

            this.assertion = null;
            final Runnable writer = new Runnable() {

                @Override
                public void run() {
                    try {
                        try {
                            // Wait that Petals CLI starts
                            Thread.sleep(1000);

                            final String output1 = outBAIS.toString();
                            final String error1 = errBAIS.toString();
                            assertTrue("An error occurs", error1.isEmpty());
                            assertFalse("Prompt is empty", output1.isEmpty());
                            assertTrue("Invalid prompt", output1.contains(Main.PROMPT_BASE + ">"));
                            assertFalse("Invalid prompt", output1.contains(expectedHostname + ":7700"));

                            // Connect
                            pos.write("connect\n".getBytes());
                            // Wait the execution of the command
                            Thread.sleep(1000);
                            final String output2 = outBAIS.toString().substring(output1.length());
                            assertFalse("Prompt is empty", output2.isEmpty());
                            assertTrue(
                                    "Confirmation message is missing",
                                    output2.contains("Would you like to connect to petals:*****@" + expectedHostname
                                            + ":7700? (y/n)"));

                            // Confirmation
                            pos.write("y\n".getBytes());
                            // Wait the execution of the command
                            Thread.sleep(1000);
                            final String output3 = outBAIS.toString().substring(output1.length() + output2.length());
                            assertFalse("Prompt is empty", output3.isEmpty());
                            assertTrue("Petals CLI is not connected",
                                    output3.contains(Main.PROMPT_BASE + "@" + expectedHostname + ":7700>"));

                        } catch (final Exception e) {
                            fail("An exception occurs: " + e.getMessage());
                        }
                    } catch (final AssertionError e) {
                        MainTest.this.assertion = e;
                    }
                }
            };

            final Runnable petalsCli = new Runnable() {

                @Override
                public void run() {

                    try {
                        final int exitCode = main
                                .run(new String[] { "-h", hostname, "-n", "7700", "-u", "petals", "-p", "petals", "-C" },
                                        MainTest.this.createBasicPreferences("localhost", 7700, "petals", "petals",
                                                null), null);

                        assertEquals("invalid exit code", 0, exitCode);
                    } catch (final AssertionError e) {
                        MainTest.this.assertion = e;
                    }

                }
            };

            final Thread petalsCliThread = new Thread(petalsCli, "petalsCliThread");
            final Thread writerThread = new Thread(writer, "writerThread");
            petalsCliThread.start();
            writerThread.start();
            writerThread.join();
            pos.write("exit\n".getBytes());
            pos.close();
            petalsCliThread.join();

            if (this.assertion != null) {
                throw this.assertion;
            }
        } finally {
            System.setOut(standardConsoleOutput);
            System.setErr(standardConsoleError);
            System.setIn(standardConsoleInput);

            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * <p>
     * Check the main entry point against an command line containing:
     * <ul>
     * <li>the flag debug is enabled,</li>
     * <li>the interactive mode,</li>
     * <li>the command to connect to an existing Petals node generating an error during the connection.,</li>
     * </ul>
     * as following:<br/>
     * <code>
     * > ./petals-cli.sh -d -C<br/>
     * petals-cli> connect -h 1.1.1.1 -n 7700 -u petals -p petals<br/>
     * petals-cli>
     * </code>
     * </p>
     * <p>
     * Note: the connection <b>is not a real</b> connection, but raises an exception on the host '
     * {@value ContainerAdministrationMock#UNKNOWN_HOST}'
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>error on the command 'connect' about the connection to an unknown host,</li>
     * <li>error stack trace is printed,</li>
     * <li>no usage is printed,</li>
     * <li>the command flow is not interrupted (because entered by the user that saw the error)</li>
     * <li>the prompt is printed after error.</li>
     * </ul>
     * </p>
     */
    @Test
    public final void testRun_Interactive_Error_01() throws IOException, InterruptedException {

        final PrintStream standardConsoleOutput = System.out;
        final PrintStream standardConsoleError = System.err;
        final InputStream standardConsoleInput = System.in;
        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            // Go on...
            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));
            final ByteArrayOutputStream outBAIS = new ByteArrayOutputStream();
            System.setOut(new PrintStream(outBAIS));
            final PipedOutputStream pos = new PipedOutputStream();
            System.setIn(new PipedInputStream(pos));

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();

            this.assertion = null;
            final Runnable writer = new Runnable() {

                @Override
                public void run() {
                    try {
                        try {
                            // Wait that Petals CLI starts
                            Thread.sleep(1000);

                            final String output1 = outBAIS.toString();
                            assertFalse("Prompt is empty", output1.isEmpty());
                            assertTrue("Petals CLI is connected", output1.contains(Main.PROMPT_BASE + ">"));
                            // Enter the command realizing the connection
                            pos.write(String.format(
                                    "connect -h " + ContainerAdministrationMock.UNKNOWN_HOST
                                            + " -n 7700 -u petals -p petals%n").getBytes());
                            // Wait the execution of the command
                            Thread.sleep(10000);
                            final String error1 = errBAIS.toString();
                            assertFalse("No error message print", error1.isEmpty());
                            assertTrue("The command is missing in error message: " + error1,
                                    error1.startsWith("ERROR on command '" + new Connect().getName() + "':"));
                            assertTrue("The exception throwing is missing in error message",
                                    error1.contains(ContainerAdministrationException.class.getName()));
                            assertTrue("Stack trace is missing in error message", error1.contains("Caused by"));
                            assertTrue("Stack trace is missing in error message", error1.contains(Main.class.getName()));

                            final String output2 = outBAIS.toString().substring(output1.length());
                            assertTrue("Petals CLI is connected", output2.endsWith(Main.PROMPT_BASE + ">"));

                        } catch (final Exception e) {
                            fail("An exception occurs: " + e.getMessage());
                        }
                    } catch (final AssertionError e) {
                        MainTest.this.assertion = e;
                    }
                }
            };

            final Runnable petalsCli = new Runnable() {
                @Override
                public void run() {

                    try {
                        final int exitCode = main.run(new String[] { "-d", "-C" }, null, null);
                        assertEquals("invalid exit code", 0, exitCode);

                    } catch (final AssertionError e) {
                        MainTest.this.assertion = e;
                    }

                }
            };

            final Thread petalsCliThread = new Thread(petalsCli, "petalsCliThread");
            final Thread writerThread = new Thread(writer, "writerThread");
            petalsCliThread.start();
            writerThread.start();
            writerThread.join();
            pos.write("exit\n".getBytes());
            pos.close();
            petalsCliThread.join();

            if (this.assertion != null) {
                throw this.assertion;
            }
        } finally {
            System.setOut(standardConsoleOutput);
            System.setErr(standardConsoleError);
            System.setIn(standardConsoleInput);

            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * <p>
     * Check the main entry point against an command line containing to check the host name in prompt:
     * <ul>
     * <li>the flag debug is disable,</li>
     * <li>connection defines on the command line with a host defined by its <b>hostname</b>,</li>
     * <li>the interactive mode,</li>
     * <li>the connection is done with 'connect' without args, and connection is confirmed,</li>
     * <li>no other command is executed.</li>
     * </ul>
     * as following:<br/>
     * <code>
     * > ./petals-cli.sh -h localhost -n 7700 -u petals -p petals -C<br/>
     * petals-cli> connect
     * petals-cli@localhost:7700>
     * </code>
     * </p>
     * <p>
     * Note: the connection <b>is not a real</b> connection.
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>the prompt displays connection information with an explicit hostname,</li>
     * <li>no error,</li>
     * <li>no error stacktrace is printed,</li>
     * <li>no usage is printed.</li>
     * </ul>
     * </p>
     */
    @Test
    public final void testRun_Interactive_ExplicitHostname_01() throws IOException, InterruptedException {

        final PrintStream standardConsoleOutput = System.out;
        final PrintStream standardConsoleError = System.err;
        final InputStream standardConsoleInput = System.in;

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            // Go on...
            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));
            final ByteArrayOutputStream outBAIS = new ByteArrayOutputStream();
            System.setOut(new PrintStream(outBAIS));
            final PipedOutputStream pos = new PipedOutputStream();
            System.setIn(new PipedInputStream(pos));

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();
            final String hostname = "localhost";
            final String expectedHostname = "localhost";

            this.assertion = null;
            final Runnable writer = new Runnable() {

                @Override
                public void run() {
                    try {
                        try {
                            // Wait that Petals CLI starts
                            Thread.sleep(1000);

                            final String output1 = outBAIS.toString();
                            assertFalse("Prompt is empty", output1.isEmpty());
                            assertTrue("Invalid prompt", output1.contains(Main.PROMPT_BASE + ">"));
                            assertFalse("Invalid prompt", output1.contains(expectedHostname + ":7700"));

                            // Connect
                            pos.write("connect\n".getBytes());
                            // Wait the execution of the command
                            Thread.sleep(1000);
                            final String output2 = outBAIS.toString().substring(output1.length());
                            assertFalse("Prompt is empty", output2.isEmpty());
                            assertTrue(
                                    "Confirmation message is missing",
                                    output2.contains("Would you like to connect to petals:*****@" + expectedHostname
                                            + ":7700? (y/n)"));

                            // Confirmation
                            pos.write("y\n".getBytes());
                            // Wait the execution of the command
                            Thread.sleep(1000);
                            final String output3 = outBAIS.toString().substring(output1.length() + output2.length());
                            assertFalse("Prompt is empty", output3.isEmpty());
                            assertTrue("Petals CLI is not connected",
                                    output3.contains(Main.PROMPT_BASE + "@" + expectedHostname + ":7700>"));

                        } catch (final Exception e) {
                            fail("An exception occurs: " + e.getMessage());
                        }
                    } catch (final AssertionError e) {
                        MainTest.this.assertion = e;
                    }
                }
            };

            final Runnable petalsCli = new Runnable() {

                @Override
                public void run() {

                    try {
                        final int exitCode = main
                                .run(new String[] { "-h", hostname, "-n", "7700", "-u", "petals", "-p", "petals", "-C" },
                                        MainTest.this.createBasicPreferences("localhost", 7700, "petals", "petals",
                                                null), null);

                        assertEquals("invalid exit code", 0, exitCode);
                    } catch (final AssertionError e) {
                        MainTest.this.assertion = e;
                    }

                }
            };

            final Thread petalsCliThread = new Thread(petalsCli, "petalsCliThread");
            final Thread writerThread = new Thread(writer, "writerThread");
            petalsCliThread.start();
            writerThread.start();
            writerThread.join();
            pos.write("exit\n".getBytes());
            pos.close();
            petalsCliThread.join();

            if (this.assertion != null) {
                throw this.assertion;
            }
        } finally {
            System.setOut(standardConsoleOutput);
            System.setErr(standardConsoleError);
            System.setIn(standardConsoleInput);

            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * <p>
     * Check the main entry point against an command line containing:
     * <ul>
     * <li>the flag debug is disable,</li>
     * <li>connection defines on the command line,</li>
     * <li>the interactive mode,</li>
     * <li>the connection is done with 'connect' without args, and connection is confirmed,</li>
     * <li>the command 'list' without arguments is executed after a 'connect'.</li>
     * </ul>
     * as following:<br/>
     * <code>
     * > ./petals-cli.sh -h localhost -n 7700 -u petals -p petals -C<br/>
     * petals-cli> connect
     * petals-cli@localhost:7700> list
     * </code>
     * </p>
     * <p>
     * Note: the connection <b>is not a real</b> connection.
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>Petals CLI is connected before entering the command,</li>
     * <li>the prompt displays connection information,</li>
     * <li>no error,</li>
     * <li>no error stacktrace is printed,</li>
     * <li>no usage is printed,</li>
     * <li>commands correctly executed.</li>
     * </ul>
     * </p>
     */
    @Test
    public final void testRun_Interactive_00() throws IOException, InterruptedException {

        final PrintStream standardConsoleOutput = System.out;
        final PrintStream standardConsoleError = System.err;
        final InputStream standardConsoleInput = System.in;

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            // Go on...
            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));
            final ByteArrayOutputStream outBAIS = new ByteArrayOutputStream();
            System.setOut(new PrintStream(outBAIS));
            final PipedOutputStream pos = new PipedOutputStream();
            System.setIn(new PipedInputStream(pos));

            // The main class MUST be instanced after to have set the console streams !!
            final Main main = new Main();
            final String hostname = "localhost";

            this.assertion = null;
            final Runnable writer = new Runnable() {

                @Override
                public void run() {
                    try {
                        try {
                            // Wait that Petals CLI starts
                            Thread.sleep(1000);

                            final String output1 = outBAIS.toString();
                            assertFalse("Prompt is empty", output1.isEmpty());
                            assertTrue("Invalid prompt", output1.contains(Main.PROMPT_BASE + ">"));
                            assertFalse("Invalid prompt", output1.contains(hostname + ":7700"));

                            // Connect
                            pos.write("connect\n".getBytes());
                            // Wait the execution of the command
                            Thread.sleep(1000);
                            final String output2 = outBAIS.toString().substring(output1.length());
                            assertFalse("Prompt is empty", output2.isEmpty());
                            assertTrue(
                                    "Confirmation message is missing",
                                    output2.contains("Would you like to connect to petals:*****@" + hostname
                                            + ":7700? (y/n)"));

                            // Confirmation
                            pos.write("y\n".getBytes());
                            // Wait the execution of the command
                            Thread.sleep(1000);
                            final String output3 = outBAIS.toString().substring(output1.length() + output2.length());
                            assertFalse("Prompt is empty", output3.isEmpty());
                            assertTrue("Petals CLI is not connected",
                                    output3.contains(Main.PROMPT_BASE + "@" + hostname + ":7700>"));

                            // Enter the command
                            pos.write("list\n".getBytes());
                            // Wait the execution of the command
                            Thread.sleep(1000);
                            final String error = errBAIS.toString();
                            assertTrue("Error message print", error.isEmpty());

                        } catch (final Exception e) {
                            fail("An exception occurs: " + e.getMessage());
                        }
                    } catch (final AssertionError e) {
                        MainTest.this.assertion = e;
                    }
                }
            };

            final Runnable petalsCli = new Runnable() {

                @Override
                public void run() {

                    try {
                        final int exitCode = main
                                .run(new String[] { "-h", hostname, "-n", "7700", "-u", "petals", "-p", "petals", "-C" },
                                        MainTest.this.createBasicPreferences("localhost", 7700, "petals", "petals",
                                                null), null);

                        assertEquals("invalid exit code", 0, exitCode);
                    } catch (final AssertionError e) {
                        MainTest.this.assertion = e;
                    }

                }
            };

            final Thread petalsCliThread = new Thread(petalsCli, "petalsCliThread");
            final Thread writerThread = new Thread(writer, "writerThread");
            petalsCliThread.start();
            writerThread.start();
            writerThread.join();
            pos.write("exit\n".getBytes());
            pos.close();
            petalsCliThread.join();

            if (this.assertion != null) {
                throw this.assertion;
            }
        } finally {
            System.setOut(standardConsoleOutput);
            System.setErr(standardConsoleError);
            System.setIn(standardConsoleInput);

            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }
}
