/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.pref;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Map;
import java.util.Properties;

import org.junit.Test;
import org.junit.experimental.categories.Category;
import org.ow2.petals.cli.api.connection.AuthenticatedConnectionParameters;
import org.ow2.petals.cli.api.connection.ConnectionParameters;
import org.ow2.petals.cli.api.pref.PreferenceFileException;
import org.ow2.petals.cli.api.pref.exception.MissingPropertyException;
import org.ow2.petals.cli.base.junit.category.UnitTestsRequiringSetEnvVar;
import org.ow2.petals.cli.connection.AuthenticatedConnectionParametersImpl;
import org.ow2.petals.cli.pref.exception.MissingPasswordException;
import org.ow2.petals.cli.pref.exception.MissingUserException;

public class PreferencesImplTest {

    /**
     * Preferences filename defined as maven-surefire-plugin and set into the environment variable
     * {@value PreferenceFileManager#PREFERENCE_ENV_VAR}
     */
    private final static String PREFERENCES_FILE_FOR_TEST = System.getProperty("PREFERENCES_FILE_FOR_TEST");

    @Test
    @Category(UnitTestsRequiringSetEnvVar.class)
    public final void testInitializePreferenceConnectionParametersWithoutErrors() throws Exception {

        final Properties preferenceProperties = new Properties();
        
        final String alias1 = "server1";
        final String hostTest1 = "localhost";
        final int portTest1 = 7458;
        preferenceProperties.setProperty(alias1 + AbstractPreferencesImpl.HOST_PROPERTY_SUFFIX, hostTest1);
        preferenceProperties.setProperty(alias1 + AbstractPreferencesImpl.PORT_PROPERTY_SUFFIX,
                String.valueOf(portTest1));

        final String alias2 = "server2";
        final String hostTest2 = "1.1.1.1";
        final int portTest2 = 7459;
        final String userTest2 = "userTest2";
        final String pwdTest2 = "pwdTest2";
        preferenceProperties.setProperty(alias2 + AbstractPreferencesImpl.HOST_PROPERTY_SUFFIX, hostTest2);
        preferenceProperties.setProperty(alias2 + AbstractPreferencesImpl.PORT_PROPERTY_SUFFIX,
                String.valueOf(portTest2));
        preferenceProperties.setProperty(alias2 + PreferencesImpl.USERNAME_PROPERTY_SUFFIX, userTest2);
        preferenceProperties.setProperty(alias2 + PreferencesImpl.PASSWORD_PROPERTY_SUFFIX, pwdTest2);

        final String alias3 = "server3";
        final String hostTest3 = "2.2.2.2";
        final int portTest3 = 7460;
        final String userTest3 = "userTest3";
        final String pwdTest3 = "pwdTest3";
        final String passPhrase3 = "passPhrase3";
        preferenceProperties.setProperty(alias3 + AbstractPreferencesImpl.HOST_PROPERTY_SUFFIX, hostTest3);
        preferenceProperties.setProperty(alias3 + AbstractPreferencesImpl.PORT_PROPERTY_SUFFIX,
                String.valueOf(portTest3));
        preferenceProperties.setProperty(alias3 + PreferencesImpl.USERNAME_PROPERTY_SUFFIX, userTest3);
        preferenceProperties.setProperty(alias3 + PreferencesImpl.PASSWORD_PROPERTY_SUFFIX, pwdTest3);
        preferenceProperties.setProperty(alias3 + PreferencesImpl.PASSPHRASE_PROPERTY_SUFFIX, passPhrase3);
        
        preferenceProperties.setProperty(AbstractPreferencesImpl.DEFAULT_ALIAS_PROPERTY, alias3);

        assertNotNull("Env var PREFERENCES_FILE_FOR_TEST no set", PREFERENCES_FILE_FOR_TEST);
        final File prefFile = new File(PREFERENCES_FILE_FOR_TEST);
        final OutputStream fos = new FileOutputStream(prefFile);
        try {
            preferenceProperties.store(fos, "");
        } finally {
            fos.close();
        }

        final AbstractPreferencesImpl pref = new PreferencesImpl();
        pref.intializePreferenceParameters();
        final Map<String, ConnectionParameters> preferenceConnectionParameters = pref
                .getPreferenceConnectionParameters();

        final ConnectionParameters connectionParameter1 = preferenceConnectionParameters.get(alias1);
        assertEquals(new ConnectionParameters(hostTest1, portTest1), connectionParameter1);

        final ConnectionParameters connectionParameter2 = preferenceConnectionParameters.get(alias2);
        assertTrue(connectionParameter2 instanceof AuthenticatedConnectionParameters);
        assertEquals(new AuthenticatedConnectionParametersImpl(hostTest2, portTest2, userTest2, pwdTest2, null),
                connectionParameter2);

        final ConnectionParameters connectionParameter3 = preferenceConnectionParameters.get(alias3);
        assertTrue(connectionParameter3 instanceof AuthenticatedConnectionParameters);
        assertEquals(new AuthenticatedConnectionParametersImpl(hostTest3, portTest3, userTest3, pwdTest3, passPhrase3),
                connectionParameter3);
    }
    
    /**
     * <p>
     * Verify the connection parameters with errors:
     * <ul>
     * <li>the password is missing instead the username is set</li>
     * <li>the username is missing instead the password is set</li>
     * <li>host and port are missing</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: errors are correctly reported
     * </p>
     */
    @Test(expected = PreferenceFileException.class)
    @Category(UnitTestsRequiringSetEnvVar.class)
    public final void testInitializePreferenceConnectionParametersWithErrors() throws IOException,
            PreferenceFileException {

        final Properties preferenceProperties = new Properties();
        
        final String alias2 = "server2";
        preferenceProperties.setProperty(alias2 + AbstractPreferencesImpl.HOST_PROPERTY_SUFFIX, "1.1.1.1");
        preferenceProperties.setProperty(alias2 + AbstractPreferencesImpl.PORT_PROPERTY_SUFFIX, "7459");
        preferenceProperties.setProperty(alias2 + PreferencesImpl.USERNAME_PROPERTY_SUFFIX, "userTest");

        final String alias3 = "server3";
        preferenceProperties.setProperty(alias3 + AbstractPreferencesImpl.HOST_PROPERTY_SUFFIX, "2.2.2.2");
        preferenceProperties.setProperty(alias3 + AbstractPreferencesImpl.PORT_PROPERTY_SUFFIX, "7457");
        preferenceProperties.setProperty(alias3 + PreferencesImpl.PASSWORD_PROPERTY_SUFFIX, "pwdTes2t");
        
        final String alias4 = "server4";
        preferenceProperties.setProperty(alias4 + PreferencesImpl.USERNAME_PROPERTY_SUFFIX, "userTest");

        assertNotNull("Env var PREFERENCES_FILE_FOR_TEST no set", PREFERENCES_FILE_FOR_TEST);
        final File prefFile = new File(PREFERENCES_FILE_FOR_TEST);
        final OutputStream fos = new FileOutputStream(prefFile);
        try {
            preferenceProperties.store(fos, "");
        } finally {
            fos.close();
        }

        try {
            final AbstractPreferencesImpl pref = new PreferencesImpl();
            pref.intializePreferenceParameters();
        } catch (final PreferenceFileException e) {
            assertTrue(e.getMessage().contains(String.format(MissingPasswordException.MESSAGE_PATTERN, alias2)));
            assertTrue(e.getMessage().contains(String.format(MissingUserException.MESSAGE_PATTERN, alias3)));
            assertTrue(e.getMessage().contains(
                    String.format(MissingPropertyException.MESSAGE_PATTERN, alias4
                            + AbstractPreferencesImpl.HOST_PROPERTY_SUFFIX)));
            assertTrue(e.getMessage().contains(
                    String.format(MissingPropertyException.MESSAGE_PATTERN, alias4
                            + AbstractPreferencesImpl.PORT_PROPERTY_SUFFIX)));
            assertTrue(e.getMessage().contains(String.format(MissingPasswordException.MESSAGE_PATTERN, alias4)));

            throw e;
        }
    }
}
