/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.junit.Assert.assertNotNull;

import java.net.URL;
import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.junit.After;
import org.junit.Before;
import org.ow2.petals.admin.PetalsAdministrationFactoryMock;
import org.ow2.petals.admin.api.ArtifactAdministration;
import org.ow2.petals.admin.api.PetalsAdministrationFactory;
import org.ow2.petals.admin.api.exception.ArtifactAdministrationException;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.MissingServiceException;
import org.ow2.petals.cli.api.connection.ConnectionParameters;
import org.ow2.petals.cli.api.connection.exception.HostMissingException;
import org.ow2.petals.cli.api.connection.exception.InvalidPortException;
import org.ow2.petals.cli.api.connection.exception.PasswordMissingException;
import org.ow2.petals.cli.api.connection.exception.UsernameMissingException;
import org.ow2.petals.cli.api.pref.PreferenceFileException;
import org.ow2.petals.cli.api.pref.Preferences;
import org.ow2.petals.cli.api.pref.exception.MissingDefaultPreferenceFileException;
import org.ow2.petals.cli.api.pref.exception.PreferenceFileNotFoundException;
import org.ow2.petals.cli.connection.AuthenticatedConnectionParametersImpl;

/**
 * Abstract class with useful method for artifact command test.
 * 
 * @author "Mathieu CARROLLE - EBM WebSourcing"
 * 
 */
public class AbstractArtifactCommandTest {

    protected final static String COMPONENT_FILENAME = "jbi-component-soap.zip";

    protected final static URL componentURL = Thread.currentThread().getContextClassLoader()
            .getResource(COMPONENT_FILENAME);

    protected final static String COMPONENT_ID = "petals-bc-soap-provide";

    protected final static String SA_PERMISSION_FILENAME = "sa-permission.zip";

    protected final static URL saURL = Thread.currentThread().getContextClassLoader()
            .getResource(SA_PERMISSION_FILENAME);

    protected final static String SA_SOAPID_FILENAME = "sa-soapid.zip";

    protected final static URL saSoapidURL = Thread.currentThread().getContextClassLoader()
            .getResource(SA_SOAPID_FILENAME);

    protected final static String SL_FILENAME = "petals-sl-mysql-5.1.5.zip";

    protected final static String SL_VERSION = "5.1.5";

    protected final URL slURL = Thread.currentThread().getContextClassLoader().getResource(SL_FILENAME);

    protected final static String SL2_FILENAME = "petals-sl-mysql-4.1.5.zip";

    protected final static String SL2_VERSION = "4.1.5";

    protected final URL sl2URL = Thread.currentThread().getContextClassLoader().getResource(SL2_FILENAME);

    protected DummyShellWrapper dummyShellWrapper;

    @Before
    public void beforeTest() {
        assertNotNull("Component archive '" + COMPONENT_FILENAME + "' not found !", this.componentURL);
        assertNotNull("SA archive '" + SA_PERMISSION_FILENAME + "' not found !", this.saURL);
        assertNotNull("SA archive '" + SA_SOAPID_FILENAME + "' not found !", this.saSoapidURL);
        assertNotNull("SL archive '" + SL_FILENAME + "' not found !", this.slURL);
        assertNotNull("SL archive '" + SL2_FILENAME + "' not found !", this.sl2URL);
        System.setProperty(PetalsAdministrationFactory.PROPERTY_NAME,
                PetalsAdministrationFactoryMock.class.getName());
    }

    @Before
    public void createAndConnectDummyShellWrapper() {
        this.dummyShellWrapper = new DummyShellWrapper();
        this.dummyShellWrapper.getShell().onConnectionEstablished();
    }

    @After
    public void afterTest() throws ArtifactAdministrationException {
        // clean registry
        System.clearProperty(PetalsAdministrationFactory.PROPERTY_NAME);
    }

    /**
     * Deploy specified artifacts.
     * 
     * @param urls
     * @throws ArtifactAdministrationException
     * @throws MissingServiceException
     * @throws DuplicatedServiceException
     */
    protected void deployStartArtifact(URL... urls) throws ArtifactAdministrationException {
        try {
            ArtifactAdministration artifactAdministration = PetalsAdministrationFactory
                    .newInstance().newArtifactAdministration();
            for (URL url : urls) {
                artifactAdministration.deployAndStartArtifact(url, false);
            }
        } catch (final DuplicatedServiceException e) {
            throw new ArtifactAdministrationException(e);
        } catch (final MissingServiceException e) {
            throw new ArtifactAdministrationException(e);
        }
    }

    protected final static String convertOptions2String(final List missingOptions) {
        final StringBuffer message = new StringBuffer();
        final Iterator itMissingOptions = missingOptions.iterator();
        while (itMissingOptions.hasNext()) {
            message.append(itMissingOptions.next());
            if (itMissingOptions.hasNext()) {
                message.append(", ");
            }
        }
        return message.toString();
    }

    /**
     * Create a basic preference configuration
     * 
     * @param host
     * @param port
     * @param username
     * @param password
     * @param passPhrase
     * @return A basic preference configuration
     */
    protected Preferences createBasicPreferences(final String host, final int port, final String username,
            final String password, final String passPhrase) {

        try {
            final ConnectionParameters connectionParameters = new AuthenticatedConnectionParametersImpl(host, port,
                    username, password, passPhrase);
            return new Preferences() {

                @Override
                public void intializePreferenceParameters() throws MissingDefaultPreferenceFileException,
                        PreferenceFileNotFoundException, PreferenceFileException {
                    // NOP
                }

                @Override
                public Map<String, ConnectionParameters> getPreferenceConnectionParameters() {
                    final Map<String, ConnectionParameters> connectionParametersMap = new HashMap<String, ConnectionParameters>();
                    connectionParametersMap.put("default", connectionParameters);
                    return connectionParametersMap;
                }

                @Override
                public ConnectionParameters getDefaultConnectionParameters() {
                    return connectionParameters;
                }
            };
        } catch (final UnknownHostException e) {
            // NOP: This exception has not to occur
            return null;
        } catch (final HostMissingException e) {
            // NOP: This exception has not to occur
            return null;
        } catch (final InvalidPortException e) {
            // NOP: This exception has not to occur
            return null;
        } catch (final UsernameMissingException e) {
            // NOP: This exception has not to occur
            return null;
        } catch (final PasswordMissingException e) {
            // NOP: This exception has not to occur
            return null;
        }
    }
}
