/**
 * Copyright (c) 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.Test;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.ConnectionRequiredException;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;

/**
 * Unit test of {@link TopologyList}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class TopologyListTest extends AbstractArtifactCommandTest {

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() throws CommandException {

        final Command command = new TopologyList();
        command.setShell(this.dummyShellWrapper.getShell());
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage", usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage", usage.contains(command.getName()));
    }

    /**
     * Test of the command "topology-list" against argument management, where:
     * <ul>
     * <li>the pass phrase is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testArgumentsError_0() throws CommandException {

        final Command command = new TopologyList();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + TopologyList.PASSPHRASE_SHORT_OPTION });
        } catch (final CommandMissingArgumentException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
            assertTrue("The option '-" + TopologyList.PASSPHRASE_SHORT_OPTION + "' has a missing argument.",
                    TopologyList.PASSPHRASE_SHORT_OPTION.equals(e.getOption().getOpt()));
            assertTrue("Missing argument is missing in error message.",
                    e.getMessage().endsWith(TopologyList.PASSPHRASE_SHORT_OPTION));
        }
    }

    /**
     * Test of the command "topology-list" against argument management, where:
     * <ul>
     * <li>the passphrase is not provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs,</li>
     * <li>not critical topology information are printed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testArgumentsError_1() throws CommandException {

        final Command command = new TopologyList();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] {});
        assertFalse("No topology information is displayed !", this.dummyShellWrapper.getOut().toString().isEmpty());
    }

    /**
     * Test of the command "topology-list" against argument management, where:
     * <ul>
     * <li>the passphrase is provided without its option.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_2() throws CommandException {

        final Command command = new TopologyList();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "a-pass-phrase" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test of the command "topology-list" against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_3() throws CommandException {

        final Command command = new TopologyList();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test of the command "topology-list" against no domain found.</p>
     * <p>
     * Expected results: an error occurs about no domain found.
     * </p>
     */
    @Test
    public void testError_0() throws CommandException {

        final Command command = new TopologyList();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + TopologyList.PASSPHRASE_SHORT_OPTION, "no-domain-found-passphrase" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandException e) {
            assertEquals("", "No Petals domain found", e.getMessage());
        }
    }

    /**
     * Test of the command "topology-list" against the option '-c' used without regexp.</p>
     * <p>
     * Expected results: no error occurs, the name of the container on which we are connected is used.
     * </p>
     */
    @Test
    public void testArguments_0() throws CommandException {

        final Command command = new TopologyList();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] { "-" + TopologyList.CONTAINER_FILTER_SHORT_OPTION });
    }

    /**
     * <p>
     * Test of the command "topology-list" when no connection is established to run the command.
     * </p>
     * <p>
     * Expected results: an error occurs about the connection not established.
     * </p>
     */
    @Test(expected = ConnectionRequiredException.class)
    public void executeNotConnected() throws CommandException {

        final StringStreamShell shell = new StringStreamShell();
        final Command command = new TopologyList();
        command.setShell(shell);

        command.execute(new String[] {});
    }
}