/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.cli.Constants.CommonOption.URL_LONG_OPTION;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;
import java.net.URL;
import java.net.UnknownHostException;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.ow2.petals.admin.api.ArtifactAdministration;
import org.ow2.petals.admin.api.PetalsAdministrationFactory;
import org.ow2.petals.admin.api.artifact.Artifact;
import org.ow2.petals.admin.api.artifact.ArtifactException;
import org.ow2.petals.admin.api.artifact.ArtifactUtils;
import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.exception.ArtifactAdministrationException;
import org.ow2.petals.admin.api.exception.ArtifactNotFoundException;
import org.ow2.petals.admin.api.exception.ArtifactTypeIsNeededException;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.MissingServiceException;
import org.ow2.petals.cli.Constants;
import org.ow2.petals.cli.Main;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.ConnectionCommand;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingYesFlagException;
import org.ow2.petals.cli.api.command.exception.ConnectionRequiredException;
import org.ow2.petals.cli.api.connection.ConnectionParameters;
import org.ow2.petals.cli.api.connection.exception.InvalidConnectionParameterException;
import org.ow2.petals.cli.api.shell.exception.ConnectionCommandAlreadyRegisteredException;
import org.ow2.petals.cli.api.shell.exception.DuplicatedCommandException;
import org.ow2.petals.cli.api.shell.exception.ShellException;
import org.ow2.petals.cli.base.junit.shell.PetalsInteractiveCliUtils;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;
import org.ow2.petals.cli.connection.AuthenticatedConnectionParametersImpl;
import org.ow2.petals.cli.shell.PetalsCli;
import org.ow2.petals.cli.shell.ShellFactory;

/**
 * 
 * @author "Mathieu CARROLLE - EBM WebSourcing"
 * 
 */
public class UndeployTest extends AbstractArtifactCommandTest {

    /**
     * The shell {@link PetalsCli} establishes automatically a connection. Here,
     * the default connection parameters.
     */
    private final ConnectionParameters defaultConnectionParameters;

    private final static String EXPECTED_CMD_USAGE = String
            .format("usage: undeploy [-a <artifact-id>] [-b <artifacts-directory>] [-t%n       <artifact-type>] [-u <url>] [-v <version>] [-y]%n");

    /**
     * Assertion exception used by threads
     */
    private volatile AssertionError assertion;

    @Rule
    public TemporaryFolder folder = new TemporaryFolder();

    public UndeployTest() {
        try {
            this.defaultConnectionParameters = new AuthenticatedConnectionParametersImpl("localhost",
                    7700, "petals", "petals", null);
        } catch (final UnknownHostException e) {
            // NOP, the exception can not occur
            throw new RuntimeException("An unexpected exception occurs", e);
        } catch (final InvalidConnectionParameterException e) {
            // NOP, the exception can not occur
            throw new RuntimeException("An unexpected exception occurs", e);
        }
    }

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() throws CommandException {

        final Undeploy command = new Undeploy();
        command.setShell(this.dummyShellWrapper.getShell());
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertEquals("Unexpected command usage", EXPECTED_CMD_USAGE, usage);
    }

    /**
     * Test of the command "undeploy" against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_00() throws CommandException {

        final Undeploy command = new Undeploy();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test of the command "undeploy" against argument management, where:
     * <ul>
     * <li>undeployment by url is provided,</li>
     * <li>bulk mode is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_01() throws CommandException {

        final Undeploy command = new Undeploy();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + Constants.CommonOption.URL_SHORT_OPTION,
                    "file:///tmp",
                    "-" + Undeploy.BULK_SHORT_OPTION });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test of the command "undeploy" against argument management, where:
     * <ul>
     * <li>undeployment by artifact is provided,</li>
     * <li>undeployment by url is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_02() throws CommandException {

        final Undeploy command = new Undeploy();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION,
                    "my-artifact", "-" + Constants.CommonOption.URL_SHORT_OPTION, "file:///tmp" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test of the command "undeploy" against argument management, where:
     * <ul>
     * <li>undeployment by artifact is provided,</li>
     * <li>bulk mode is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_03() throws CommandException {

        final Undeploy command = new Undeploy();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION,
                    "my-artifact", "-" + Undeploy.BULK_SHORT_OPTION });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test of the command "undeploy" against an undeployment of all artifacts,
     * where:
     * <ul>
     * <li>bulk mode enable,</li>
     * <li>no 'yes' flag,</li>
     * <li>using a non interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing yes flag.
     * </p>
     */
    @Test()
    public void testUndeployBulkError_00() throws DuplicatedCommandException, IllegalArgumentException,
            CommandException, ConnectionCommandAlreadyRegisteredException {

        final PrintStream standardConsoleError = System.err;

        try {
            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));

            final Undeploy command = new Undeploy();
            final PetalsCli shell = new PetalsCli(new String[] { command.getName(), "-" + Undeploy.BULK_SHORT_OPTION },
                    false, false, null, null, null);
            shell.registersCommand(command);
            command.setShell(shell);
            final ConnectionCommand connect = new Connect();
            connect.setShell(shell);
            shell.registersConnectionCommand(connect);

            shell.setConnectionParameters(this.defaultConnectionParameters);
            shell.run();

            final String error = errBAIS.toString();
            assertFalse("No error message print", error.isEmpty());
            assertTrue("It's not the expected error message.",
                    error.contains(CommandMissingYesFlagException.MISSING_YESFLAG));

        } finally {
            System.setErr(standardConsoleError);
        }
    }

    /**
     * Test of the command "undeploy" against an undeployment of all artifacts,
     * where:
     * <ul>
     * <li>bulk mode enable,</li>
     * <li>with the yes flag at command line level,</li>
     * <li>using a non interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: No error occurs about missing yes flag
     * </p>
     */
    @Test
    public void testUndeployBulk_00() throws ArtifactAdministrationException,
            DuplicatedServiceException, MissingServiceException, CommandException {

        final PrintStream standardConsoleError = System.err;
        final ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();

        try {
            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));

            // Set the Petals ESB environment
            this.deployStartArtifact(componentURL, saURL);
            assertEquals("Unexpected deployed artifact number", 2, admin.listArtifacts().size());

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();

            final int exitCode = main.run(new String[] { "-" + Connect.HOST_SHORT_OPTION, "localhost",
                    "-" + Connect.PORT_SHORT_OPTION, "7700", "-" + Connect.USER_SHORT_OPTION, "petals",
                    "-" + Connect.PWD_SHORT_OPTION, "petals", "-" + ShellFactory.YESFLAG_SHORT_OPTION,
                    "-" + ShellFactory.COMMAND_SHORT_OPTION, "--", new Undeploy().getName(),
                    "-" + Undeploy.BULK_SHORT_OPTION }, null, null);

            final String error = errBAIS.toString();
            assertEquals("invalid exit code", 0, exitCode);
            assertTrue("An error message is print", error.isEmpty());
            assertEquals("Artifact is always deployed", 0, admin.listArtifacts().size());

        } finally {
            admin.stopAndUndeployAllArtifacts();
            System.setErr(standardConsoleError);
        }
    }

    /**
     * Test of the command "undeploy" against an undeployment of all artifacts,
     * where:
     * <ul>
     * <li>bulk mode enable,</li>
     * <li>with the yes flag at command level,</li>
     * <li>using a non interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: No error occurs about missing yes flag
     * </p>
     */
    @Test
    public void testUndeployBulk_01() throws ArtifactAdministrationException,
            DuplicatedServiceException, MissingServiceException, CommandException {

        final PrintStream standardConsoleError = System.err;
        final ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();

        try {
            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));

            // Set the Petals ESB environment
            this.deployStartArtifact(componentURL, saURL);
            assertEquals("Unexpected deployed artifact number", 2, admin.listArtifacts().size());

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();

            final int exitCode = main.run(new String[] { "-" + Connect.HOST_SHORT_OPTION, "localhost",
                    "-" + Connect.PORT_SHORT_OPTION, "7700", "-" + Connect.USER_SHORT_OPTION, "petals",
                    "-" + Connect.PWD_SHORT_OPTION, "petals", "-" + ShellFactory.COMMAND_SHORT_OPTION, "--",
                    new Undeploy().getName(), "-" + Undeploy.BULK_SHORT_OPTION, "-" + Undeploy.YESFLAG_SHORT_OPTION, },
                    null, null);

            final String error = errBAIS.toString();
            assertEquals("invalid exit code", 0, exitCode);
            assertTrue("An error message is print", error.isEmpty());
            assertEquals("Artifact is always deployed", 0, admin.listArtifacts().size());

        } finally {
            admin.stopAndUndeployAllArtifacts();
            System.setErr(standardConsoleError);
        }
    }

    /**
     * Test of the command "undeploy" against an undeployment of all artifacts,
     * where:
     * <ul>
     * <li>bulk mode enable,</li>
     * <li>without yes flag,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>a confirmation message is displayed,</li>
     * <li>after confirmation, the command is executed displaying the prompt</li>
     * </ul>
     * </p>
     */
    @Test
    public void testUndeployBulkError_02() throws CommandException, DuplicatedCommandException,
            IllegalArgumentException, InterruptedException, IOException,
            ArtifactAdministrationException, DuplicatedServiceException, MissingServiceException {

        final PrintStream standardConsoleOutput = System.out;
        final PrintStream standardConsoleError = System.err;
        final InputStream standardConsoleInput = System.in;

        final ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();
        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));
            final ByteArrayOutputStream outBAIS = new ByteArrayOutputStream();
            System.setOut(new PrintStream(outBAIS));
            final PipedOutputStream pos = new PipedOutputStream();
            System.setIn(new PipedInputStream(pos));

            // Set the Petals ESB environment
            this.deployStartArtifact(componentURL, saURL);
            assertEquals("Unexpected deployed artifact number", 2, admin.listArtifacts().size());

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();
            this.assertion = null;
            final Runnable writer = new Runnable() {

                @Override
                public void run() {
                    try {
                        try {
                            // Wait that Petals CLI starts
                            Thread.sleep(1000);

                            final String output1 = outBAIS.toString();
                            assertFalse("Prompt is empty", output1.isEmpty());
                            assertTrue("Petals CLI is connected", output1.contains("petals-cli>"));

                            // Connect
                            pos.write("connect -h localhost -n 7700 -u petals -p petals\n".getBytes());
                            // Wait the execution of the command
                            Thread.sleep(1000);
                            final String output2 = outBAIS.toString().substring(output1.length());
                            final String error0 = errBAIS.toString();
                            assertTrue("An error message is print", error0.isEmpty());
                            assertFalse("Prompt is empty", output2.isEmpty());
                            assertTrue("Petals CLI is not connected",
                                    output2.contains(Main.PROMPT_BASE + "@localhost:7700>"));

                            // Enter the command
                            pos.write(String.format("%s -b%n", new Undeploy().getName()).getBytes());
                            pos.flush();
                            // Wait the execution of the command
                            Thread.sleep(250);
                            final String error1 = errBAIS.toString();
                            final String output3 = outBAIS.toString().substring(output1.length() + output2.length());
                            assertTrue("An error message is print", error1.isEmpty());
                            assertFalse("No confirmation message print", output3.isEmpty());
                            assertTrue("Unexpected confirmation message",
                                    output3.endsWith(Undeploy.UNDEPLOY_ALL_CONFIRMATION_MSG));

                            // Enter confirmation
                            pos.write(String.format("y%n").getBytes());
                            pos.flush();
                            // Wait the execution of the command
                            Thread.sleep(250);
                            final String error2 = errBAIS.toString().substring(error1.length());
                            final String output4 = outBAIS.toString().substring(
                                    output1.length() + output2.length() + output3.length());
                            assertTrue("An error message is print", error2.isEmpty());
                            assertTrue("Unexpected prompt", output4.endsWith(Main.PROMPT_BASE + "@localhost:7700>"));

                        } catch (final Exception e) {
                            fail("An exception occurs: " + e.getMessage());
                        }
                    } catch (final AssertionError e) {
                        assertion = e;
                    }
                }
            };

            final Runnable petalsCli = new Runnable() {

                @Override
                public void run() {

                    try {
                        final int exitCode = main.run(new String[] { "-C" }, null, null);

                        assertEquals("invalid exit code", 0, exitCode);
                    } catch (final AssertionError e) {
                        assertion = e;
                    }

                }
            };

            final Thread petalsCliThread = new Thread(petalsCli, "petalsCliThread");
            final Thread writerThread = new Thread(writer, "writerThread");
            petalsCliThread.start();
            writerThread.start();
            writerThread.join();
            pos.write(String.format("exit%n").getBytes());
            pos.close();
            petalsCliThread.join();

            if (this.assertion != null) {
                throw this.assertion;
            }
            assertEquals("Artifact is always deployed", 0, admin.listArtifacts().size());
        } finally {
            admin.stopAndUndeployAllArtifacts();

            System.setOut(standardConsoleOutput);
            System.setErr(standardConsoleError);
            System.setIn(standardConsoleInput);

            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * Test of the command "undeploy" against an undeployment of all artifacts,
     * where:
     * <ul>
     * <li>bulk mode enable,</li>
     * <li>with the yes flag at command line level,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no confirmation message is displayed,</li>
     * <li>the command is executed displaying the prompt</li>
     * </ul>
     * </p>
     */
    @Test
    public void testUndeployBulk_02() throws CommandException, DuplicatedCommandException,
            IllegalArgumentException, InterruptedException, IOException,
            ArtifactAdministrationException, DuplicatedServiceException, MissingServiceException {

        final PrintStream standardConsoleOutput = System.out;
        final PrintStream standardConsoleError = System.err;
        final InputStream standardConsoleInput = System.in;

        final ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();

        try {

            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));
            final ByteArrayOutputStream outBAIS = new ByteArrayOutputStream();
            System.setOut(new PrintStream(outBAIS));
            final PipedOutputStream pos = new PipedOutputStream();
            System.setIn(new PipedInputStream(pos));

            // Set the Petals ESB environment
            this.deployStartArtifact(componentURL, saURL);
            assertEquals("Unexpected deployed artifact number", 2, admin.listArtifacts().size());

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();
            this.assertion = null;
            final Runnable writer = new Runnable() {

                @Override
                public void run() {
                    try {
                        try {
                            // Wait that Petals CLI starts
                            Thread.sleep(1000);

                            final String output1 = outBAIS.toString();
                            assertFalse("Prompt is empty", output1.isEmpty());
                            assertTrue("Petals CLI is connected", output1.contains("petals-cli>"));

                            // Connect
                            pos.write("connect -h localhost -n 7700 -u petals -p petals\n".getBytes());
                            // Wait the execution of the command
                            Thread.sleep(1000);
                            final String output2 = outBAIS.toString().substring(output1.length());
                            final String error0 = errBAIS.toString();
                            assertTrue("An error message is print", error0.isEmpty());
                            assertFalse("Prompt is empty", output2.isEmpty());
                            assertTrue("Petals CLI is not connected",
                                    output2.contains(Main.PROMPT_BASE + "@localhost:7700>"));

                            // Enter the command
                            pos.write(String.format("%s -b%n", new Undeploy().getName()).getBytes());
                            // Wait the execution of the command
                            Thread.sleep(1000);
                            final String error1 = errBAIS.toString();
                            final String output3 = outBAIS.toString().substring(output1.length() + output2.length());
                            assertTrue("An error message is print", error1.isEmpty());
                            assertTrue(
                                    "Unexpected prompt (probably a confirmation message is printed)",
                                    output3.endsWith(Main.PROMPT_BASE + "@localhost:7700>"));

                        } catch (final Exception e) {
                            fail("An exception occurs: " + e.getMessage());
                        }
                    } catch (final AssertionError e) {
                        assertion = e;
                    }
                }
            };

            final Runnable petalsCli = new Runnable() {

                @Override
                public void run() {

                    try {
                        final int exitCode = main.run(new String[] { "-" + ShellFactory.YESFLAG_SHORT_OPTION, "-C" },
                                null, null);

                        assertEquals("invalid exit code", 0, exitCode);
                    } catch (final AssertionError e) {
                        assertion = e;
                    }

                }
            };

            final Thread petalsCliThread = new Thread(petalsCli, "petalsCliThread");
            final Thread writerThread = new Thread(writer, "writerThread");
            petalsCliThread.start();
            writerThread.start();
            writerThread.join();
            pos.write(String.format("exit%n").getBytes());
            pos.close();
            petalsCliThread.join();

            if (this.assertion != null) {
                throw this.assertion;
            }
            assertEquals("Artifact is always deployed", 0, admin.listArtifacts().size());
        } finally {
            admin.stopAndUndeployAllArtifacts();

            System.setOut(standardConsoleOutput);
            System.setErr(standardConsoleError);
            System.setIn(standardConsoleInput);

            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * Test of the command "undeploy" against an undeployment of all artifacts,
     * where:
     * <ul>
     * <li>bulk mode enable,</li>
     * <li>with the yes flag at command level,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no confirmation message is displayed,</li>
     * <li>the command is executed displaying the prompt,</li>
     * <li>no more artifacts remain deployed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testUndeployBulk_03() throws CommandException, DuplicatedCommandException,
            IllegalArgumentException, InterruptedException, IOException,
            ArtifactAdministrationException, DuplicatedServiceException, MissingServiceException {

        final PrintStream standardConsoleOutput = System.out;
        final PrintStream standardConsoleError = System.err;
        final InputStream standardConsoleInput = System.in;

        final ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();

        try {

            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));
            final ByteArrayOutputStream outBAIS = new ByteArrayOutputStream();
            System.setOut(new PrintStream(outBAIS));
            final PipedOutputStream pos = new PipedOutputStream();
            System.setIn(new PipedInputStream(pos));

            // Set the Petals ESB environment
            this.deployStartArtifact(componentURL, saURL);
            assertEquals("Unexpected deployed artifact number", 2, admin.listArtifacts().size());

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();
            this.assertion = null;
            final Runnable writer = new Runnable() {

                @Override
                public void run() {
                    try {
                        try {
                            // Wait that Petals CLI starts
                            Thread.sleep(1000);

                            final String output1 = outBAIS.toString();
                            assertFalse("Prompt is empty", output1.isEmpty());
                            assertTrue("Petals CLI is connected", output1.contains("petals-cli>"));

                            // Connect
                            pos.write("connect -h localhost -n 7700 -u petals -p petals\n".getBytes());
                            // Wait the execution of the command
                            Thread.sleep(1000);
                            final String output2 = outBAIS.toString().substring(output1.length());
                            final String error0 = errBAIS.toString();
                            assertTrue("An error message is print", error0.isEmpty());
                            assertFalse("Prompt is empty", output2.isEmpty());
                            assertTrue("Petals CLI is not connected",
                                    output2.contains(Main.PROMPT_BASE + "@localhost:7700>"));

                            // Enter the command
                            pos.write(String.format("%s -b -y%n", new Undeploy().getName())
                                    .getBytes());
                            // Wait the execution of the command
                            Thread.sleep(1000);
                            final String error1 = errBAIS.toString();
                            final String output3 = outBAIS.toString().substring(output1.length() + output2.length());
                            assertTrue("An error message is print", error1.isEmpty());
                            assertTrue(
                                    "Unexpected prompt (probably a confirmation message is printed)",
                                    output3.endsWith(Main.PROMPT_BASE + "@localhost:7700>"));

                        } catch (final Exception e) {
                            fail("An exception occurs: " + e.getMessage());
                        }
                    } catch (final AssertionError e) {
                        assertion = e;
                    }
                }
            };

            final Runnable petalsCli = new Runnable() {

                @Override
                public void run() {

                    try {
                        final int exitCode = main.run(new String[] { "-C" }, null, null);

                        assertEquals("invalid exit code", 0, exitCode);
                    } catch (final AssertionError e) {
                        assertion = e;
                    }

                }
            };

            final Thread petalsCliThread = new Thread(petalsCli, "petalsCliThread");
            final Thread writerThread = new Thread(writer, "writerThread");
            petalsCliThread.start();
            writerThread.start();
            writerThread.join();
            pos.write(String.format("exit%n").getBytes());
            pos.close();
            petalsCliThread.join();

            if (this.assertion != null) {
                throw this.assertion;
            }
            assertEquals("Artifact is always deployed", 0, admin.listArtifacts().size());

        } finally {
            admin.stopAndUndeployAllArtifacts();

            System.setOut(standardConsoleOutput);
            System.setErr(standardConsoleError);
            System.setIn(standardConsoleInput);

            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * Test of the command "undeploy" against an undeployment of all artifacts,
     * where:
     * <ul>
     * <li>bulk mode enable using a directory url,</li>
     * <li>with the yes flag at command level,</li>
     * <li>using a non interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: Specified artifacts are correctly undeployed
     * </p>
     */
    @Test
    public void testUndeployBulk_04() throws ArtifactAdministrationException, IOException,
            DuplicatedServiceException, MissingServiceException, CommandException {

        final PrintStream standardConsoleError = System.err;

        final ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();

        try {
            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));

            // Set the Petals ESB environment
            final URL jbiCompXslt = Thread.currentThread().getContextClassLoader()
                    .getResource("jbi-component-xslt.zip");
            assertNotNull("Test component XSLT not found", jbiCompXslt);
            this.deployStartArtifact(componentURL, saURL, jbiCompXslt);
            assertEquals("Unexpected deployed artifact number", 3, admin.listArtifacts().size());

            // Create a directory to undeploy a list of artifacts
            final File undeploymentDir = this.folder.newFolder("undeployment-dir");
            FileUtils.copyURLToFile(componentURL, new File(undeploymentDir, COMPONENT_FILENAME));
            FileUtils.copyURLToFile(saURL, new File(undeploymentDir, SA_PERMISSION_FILENAME));

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();

            final int exitCode = main.run(new String[] { "-" + Connect.HOST_SHORT_OPTION, "localhost",
                    "-" + Connect.PORT_SHORT_OPTION, "7700", "-" + Connect.USER_SHORT_OPTION, "petals",
                    "-" + Connect.PWD_SHORT_OPTION, "petals", "-" + ShellFactory.COMMAND_SHORT_OPTION, "--",
                    new Undeploy().getName(), "-" + Undeploy.BULK_SHORT_OPTION,
                    undeploymentDir.toURI().toURL().toString(), "-" + Undeploy.YESFLAG_SHORT_OPTION, }, null, null);

            final String error = errBAIS.toString();
            assertEquals("invalid exit code", 0, exitCode);
            assertTrue("An error message is print", error.isEmpty());
            final List<Artifact> artifactsList = admin.listArtifacts();
            assertEquals("It should remain 1 artifact deployed", 1, artifactsList.size());
            final Artifact artifact = artifactsList.get(0);
            assertEquals("Unexpected remaining artifact", "petals-se-xslt", artifact.getName());

        } finally {
            admin.stopAndUndeployAllArtifacts();

            System.setErr(standardConsoleError);
        }
    }

    @Test()
    public void testUndeployOneArtifactsByURL() throws ShellException,
            ArtifactAdministrationException, CommandException, DuplicatedServiceException,
            MissingServiceException {
        final ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();
        this.deployStartArtifact(componentURL, saURL);
        try {
            Undeploy command = new Undeploy();
            command.setShell(this.dummyShellWrapper.getShell());
            command.execute(new String[] { "--" + URL_LONG_OPTION, saURL.toString() });
            List<Artifact> artifacts = admin.listArtifacts();
            assertEquals(1, artifacts.size());
            assertTrue(artifacts.get(0) instanceof Component);
        } finally {
            admin.stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "undeploy" to undeploy an artifact identified by
     * artifact type and artifact identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is provided</li>
     * <li>two JBI artifacts are deployed with different identifiers.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs</li>
     * <li>artifact is correctly undeployed</li>
     * </ul>
     * </p>
     */
    @Test()
    public void testUndeployOneArtifactsByTypeAndName_0() throws ShellException,
            ArtifactAdministrationException, CommandException, DuplicatedServiceException,
            MissingServiceException {
        final ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();
        this.deployStartArtifact(componentURL, saURL);
        try {
            Artifact a = admin.getArtifactInfo(saURL);
            Undeploy command = new Undeploy();
            command.setShell(this.dummyShellWrapper.getShell());
            command.execute(new String[] { "--" + ArtifactBasedCommand.ARTIFACT_LONG_OPTION, a.getName(),
                    "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, a.getType() });
            List<Artifact> artifacts = admin.listArtifacts();
            assertEquals(1, artifacts.size());
            assertTrue(artifacts.get(0) instanceof Component);
        } finally {
            admin.stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "undeploy" to undeploy an artifact identified by
     * artifact type and artifact identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is missing</li>
     * <li>two JBI artifacts are deployed with different identifiers.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs</li>
     * <li>artifact is correctly undeploy</li>
     * </ul>
     * </p>
     */
    @Test()
    public void testUndeployOneArtifactsByTypeAndName_1() throws ShellException,
            ArtifactAdministrationException, CommandException, DuplicatedServiceException,
            MissingServiceException {
        final ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();
        this.deployStartArtifact(componentURL, saURL);
        try {
            Artifact a = admin.getArtifactInfo(saURL);
            Undeploy command = new Undeploy();
            command.setShell(this.dummyShellWrapper.getShell());
            command.execute(new String[] { "--" + ArtifactBasedCommand.ARTIFACT_LONG_OPTION, a.getName() });
            List<Artifact> artifacts = admin.listArtifacts();
            assertEquals(1, artifacts.size());
            assertTrue(artifacts.get(0) instanceof Component);
        } finally {
            admin.stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "undeploy" to undeploy an artifact identified by its
     * type and its identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is missing,</li>
     * <li>two JBI artifacts are deployed with the same identifier,</li>
     * <li>one JBI artifact is deployed with a different identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error occurs about the missing artifact type</li>
     * <li>the artifacts are not undeployed</li>
     * </ul>
     * </p>
     */
    @Test()
    public void testUndeployOneArtifactsByTypeAndName_2() throws ShellException,
            ArtifactAdministrationException, DuplicatedServiceException, MissingServiceException,
            CommandException {
        final ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();
        this.deployStartArtifact(componentURL, saURL, saSoapidURL);
        try {
            Artifact a = admin.getArtifactInfo(componentURL);
            Undeploy command = new Undeploy();
            command.setShell(this.dummyShellWrapper.getShell());
            try {
                command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, a.getName() });
                fail("ArtifactTypeIsNeededException is not thrown through CommandException.");
            } catch (final CommandException e) {
                assertNotNull("CommandException cause is missing.", e.getCause());
                assertTrue("CommandException cause is not ArtifactTypeIsNeededException",
                        e.getCause() instanceof ArtifactTypeIsNeededException);
            }

            List<Artifact> artifacts = admin.listArtifacts();
            assertEquals(3, artifacts.size());
        } finally {
            admin.stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "undeploy" to undeploy an artifact identified by its
     * type and its identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is provided,</li>
     * <li>two JBI artifacts are deployed with the same identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs</li>
     * <li>the right artifact is undeployed</li>
     * <li>the other artifact is not undeployed</li>
     * </ul>
     * </p>
     */
    @Test()
    public void testUndeployOneArtifactsByTypeAndName_3() throws ShellException,
            ArtifactAdministrationException, CommandException, DuplicatedServiceException,
            MissingServiceException {
        ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();
        this.deployStartArtifact(componentURL, saSoapidURL);
        try {
            Artifact a = admin.getArtifactInfo(saSoapidURL);
            Undeploy command = new Undeploy();
            command.setShell(this.dummyShellWrapper.getShell());
            command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, a.getName(),
                    "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, a.getType() });
            List<Artifact> artifacts = admin.listArtifacts();
            assertEquals(1, artifacts.size());
            assertTrue(artifacts.get(0) instanceof Component);
        } finally {
            admin.stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "undeploy" to undeploy an artifact identified by its type and its identifier where:
     * <ul>
     * <li>the artifact identifier is missing,</li>
     * <li>the artifact type is provided,</li>
     * <li>two JBI artifacts are deployed with the same identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error occurs about bad argument number</li>
     * <li>the artifacts are always deployed</li>
     * </ul>
     * </p>
     */
    @Test(expected = CommandBadArgumentNumberException.class)
    public void testUndeployOneArtifactsByTypeAndName_4() throws ShellException,
            ArtifactAdministrationException, CommandException, DuplicatedServiceException,
            MissingServiceException {
        ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();
        this.deployStartArtifact(componentURL, saSoapidURL);
        try {
            Artifact a = admin.getArtifactInfo(saSoapidURL);
            Undeploy command = new Undeploy();
            command.setShell(this.dummyShellWrapper.getShell());
            try {
                command.execute(new String[] { "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, a.getType() });
                fail("ArtifactNotFoundException is not thrown through CommandException.");
            } catch (final CommandBadArgumentNumberException e) {
                // Expected exception
                List<Artifact> artifacts = admin.listArtifacts();
                assertEquals(2, artifacts.size());
                throw e;
            }
        } finally {
            admin.stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "undeploy" to undeploy an artifact identified by its
     * type and its identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>a bad artifact type is provided,</li>
     * <li>two JBI artifacts are deployed with the same identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error occurs about artifact not found</li>
     * <li>the artifacts are always deployed</li>
     * </ul>
     * </p>
     */
    @Test()
    public void testUndeployOneArtifactsByTypeAndName_5() throws ShellException,
            ArtifactAdministrationException, CommandException, DuplicatedServiceException,
            MissingServiceException {
        ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();
        this.deployStartArtifact(componentURL, saSoapidURL);
        try {
            Artifact a = admin.getArtifactInfo(saSoapidURL);
            Undeploy command = new Undeploy();
            command.setShell(this.dummyShellWrapper.getShell());
            try {
                assertFalse("The artifact is a SE", "SE".equals(a.getType()));
                command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, a.getName(),
                        "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, "SE" });
                fail("ArtifactNotFoundException is not thrown through CommandException.");
            } catch (final CommandException e) {
                assertNotNull("CommandException cause is missing.", e.getCause());
                assertTrue("CommandException cause is not ArtifactNotFoundException",
                        e.getCause() instanceof ArtifactNotFoundException);
            }
            List<Artifact> artifacts = admin.listArtifacts();
            assertEquals(2, artifacts.size());
        } finally {
            admin.stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "undeploy" to undeploy an artifact identified by
     * artifact type and artifact identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>a bad artifact type is provided,</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error occurs about artifact not found</li>
     * <li>the artifact is always deployed</li>
     * </ul>
     * </p>
     */
    @Test()
    public void testUndeployOneArtifactsByTypeAndName_6() throws ShellException,
            ArtifactAdministrationException, DuplicatedServiceException, MissingServiceException,
            CommandException {
        ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();
        this.deployStartArtifact(componentURL, saURL);
        try {
            Artifact a = admin.getArtifactInfo(saURL);
            Undeploy command = new Undeploy();
            command.setShell(this.dummyShellWrapper.getShell());
            try {
                assertFalse("The artifact is a SE", "SE".equals(a.getType()));
                command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, a.getName(),
                        "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, "SE" });
                fail("ArtifactNotFoundException is not thrown through CommandException.");
            } catch (final CommandException e) {
                assertNotNull("CommandException cause is missing.", e.getCause());
                assertTrue("CommandException cause is not ArtifactNotFoundException",
                        e.getCause() instanceof ArtifactNotFoundException);
            }
            List<Artifact> artifacts = admin.listArtifacts();
            assertEquals(2, artifacts.size());
        } finally {
            admin.stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "undeploy" to undeploy an artifact identified by artifact type and artifact identifier where:
     * <ul>
     * <li>the artifact identifier is missing,</li>
     * <li>the artifact type is provided,</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error occurs about bad argument number</li>
     * <li>the artifact is always deployed</li>
     * </ul>
     * </p>
     */
    @Test(expected = CommandBadArgumentNumberException.class)
    public void testUndeployOneArtifactsByTypeAndName_7() throws ShellException,
            ArtifactAdministrationException, DuplicatedServiceException, MissingServiceException,
            CommandException {
        ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();
        this.deployStartArtifact(componentURL, saURL);
        try {
            Artifact a = admin.getArtifactInfo(saURL);
            Undeploy command = new Undeploy();
            command.setShell(this.dummyShellWrapper.getShell());
            try {
                command.execute(new String[] { "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, a.getType() });
                fail("ArtifactNotFoundException is not thrown through CommandException.");
            } catch (final CommandBadArgumentNumberException e) {
                // Expected exception
                List<Artifact> artifacts = admin.listArtifacts();
                assertEquals(2, artifacts.size());
                throw e;
            }
        } finally {
            admin.stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "undeploy" to undeploy an artifact identified by artifact type and artifact identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>a artifact type option is provided without value,</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about artifact not found
     * </p>
     */
    @Test
    public void testUndeployOneArtifactsByTypeAndName_8() throws ArtifactAdministrationException, ArtifactException,
            CommandException, DuplicatedServiceException, MissingServiceException {
        this.deployStartArtifact(this.componentURL);
        try {
            Component a = (Component) ArtifactUtils.createArtifact(this.componentURL);

            Undeploy command = new Undeploy();
            command.setShell(this.dummyShellWrapper.getShell());
            String identifier = a.getName();
            try {
                assertFalse("The artifact is a SE", "SE".equals(a.getType()));
                command.execute(new String[] { "-" + Undeploy.ARTIFACT_SHORT_OPTION, identifier,
                        "-" + Undeploy.TYPE_SHORT_OPTION });
                fail("CommandMissingArgumentException is not thrown");
            } catch (final CommandMissingArgumentException e) {
                assertTrue("Error label is missing.",
                        e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
                assertTrue("The option '-" + Undeploy.TYPE_SHORT_OPTION + "' has a missing argument.",
                        Undeploy.TYPE_SHORT_OPTION.equals(e.getOption().getOpt()));
                assertTrue("Missing argument is missing in error message.",
                        e.getMessage().endsWith(Undeploy.TYPE_SHORT_OPTION));
            }
        } finally {
            PetalsAdministrationFactory.newInstance().newArtifactAdministration().stopAndUndeployAllArtifacts();
        }
    }

    @Test
    public void testUndeploySharedLibraryWithSLVersionFilter() throws CommandException,
            ArtifactAdministrationException, ArtifactException, DuplicatedServiceException, MissingServiceException {
        this.deployStartArtifact(slURL, sl2URL, componentURL);

        try {
            final Artifact slArtifact = ArtifactUtils.createArtifact(this.slURL);
            final Undeploy command = new Undeploy();
            command.setShell(this.dummyShellWrapper.getShell());
            command.execute(new String[] { "-" + Undeploy.ARTIFACT_SHORT_OPTION, slArtifact.getName(),
                    "-" + Undeploy.VERSION_SHORT_OPTION, slArtifact.getVersion() });

            final ArtifactAdministration admin = PetalsAdministrationFactory.newInstance().newArtifactAdministration();
            final List<Artifact> artifacts = admin.listArtifacts();
            assertEquals(2, artifacts.size());
        } finally {
            PetalsAdministrationFactory.newInstance().newArtifactAdministration().stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "undeploy" when no connection is established to run the command.
     * </p>
     * <p>
     * Expected results: an error occurs about the connection not established.
     * </p>
     */
    @Test(expected = ConnectionRequiredException.class)
    public void executeNotConnected() throws CommandException {

        final StringStreamShell shell = new StringStreamShell();
        final Command command = new Undeploy();
        command.setShell(shell);

        command.execute(new String[] {});
    }
}
