/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.completer;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import jline.console.completer.Completer;

import org.junit.Rule;
import org.junit.Test;
import org.ow2.petals.admin.api.artifact.ArtifactState.State;
import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.artifact.Component.ComponentType;
import org.ow2.petals.admin.api.artifact.ServiceAssembly;
import org.ow2.petals.admin.api.artifact.ServiceUnit;
import org.ow2.petals.admin.api.artifact.SharedLibrary;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.MissingServiceException;
import org.ow2.petals.admin.junit.PetalsAdministrationApi;

/**
 * Unit tests of {@link AllArtifactNameCompleter}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ArtifactNameCompleterTest {

    @Rule
    public PetalsAdministrationApi petalsAdminApi = new PetalsAdministrationApi();

    @Test
    public void testComplete() throws DuplicatedServiceException, MissingServiceException {

        final String slOneName = "shared-library";
        final String slOneVersion = "shared-library-version";
        this.petalsAdminApi.registerArtifact(new SharedLibrary(slOneName, slOneVersion));

        final String componentOneName = "started-binding-component";
        final Component componentOne = new Component(componentOneName, ComponentType.BC);
        componentOne.setState(State.STARTED);
        this.petalsAdminApi.registerArtifact(componentOne);

        final String componentTwoName = "stopped-binding-component";
        final Component componentTwo = new Component(componentTwoName, ComponentType.SE);
        componentTwo.setState(State.STOPPED);
        this.petalsAdminApi.registerArtifact(componentTwo);

        final String componentThreeName = "loaded-binding-component";
        final Component componentThree = new Component(componentThreeName, ComponentType.BC);
        componentThree.setState(State.LOADED);
        this.petalsAdminApi.registerArtifact(componentThree);

        final String componentFourName = "shutdown-binding-component";
        final Component componentFour = new Component(componentFourName, ComponentType.BC);
        componentFour.setState(State.SHUTDOWN);
        this.petalsAdminApi.registerArtifact(componentFour);

        final String componentFiveName = "unknown-binding-component";
        final Component componentFive = new Component(componentFiveName, ComponentType.BC);
        componentFive.setState(State.UNKNOWN);
        this.petalsAdminApi.registerArtifact(componentFive);

        final String saOneName = "started-service-assembly";
        final ServiceAssembly saOne = new ServiceAssembly(saOneName);
        saOne.setState(State.STARTED);
        this.petalsAdminApi.registerArtifact(saOne);

        final String saTwoName = "stopped-service-assembly";
        final ServiceAssembly saTwo = new ServiceAssembly(saTwoName);
        saTwo.setState(State.STOPPED);
        this.petalsAdminApi.registerArtifact(saTwo);

        final String saThreeName = "unknown-service-assembly";
        final ServiceAssembly saThree = new ServiceAssembly(saThreeName);
        saThree.setState(State.UNKNOWN);
        this.petalsAdminApi.registerArtifact(saThree);

        final String suOneName = "service-unit";
        final ServiceUnit suOne = new ServiceUnit(suOneName, componentOneName);
        this.petalsAdminApi.registerArtifact(suOne);

        final List<CharSequence> allArtifacts = new ArrayList<CharSequence>();
        allArtifacts.add(slOneName);
        allArtifacts.add(componentOneName);
        allArtifacts.add(componentTwoName);
        allArtifacts.add(componentThreeName);
        allArtifacts.add(componentFourName);
        allArtifacts.add(componentFiveName);
        allArtifacts.add(saOneName);
        allArtifacts.add(saTwoName);
        allArtifacts.add(saThreeName);
        allArtifacts.add(suOneName);

        final List<CharSequence> artifactsThatCanBeStopped = new ArrayList<CharSequence>();
        artifactsThatCanBeStopped.add(componentOneName);
        artifactsThatCanBeStopped.add(saOneName);

        final List<CharSequence> artifactsThatCanBeStarted = new ArrayList<CharSequence>();
        artifactsThatCanBeStarted.add(componentTwoName);
        artifactsThatCanBeStarted.add(componentFourName);
        artifactsThatCanBeStarted.add(saTwoName);

        final List<CharSequence> candidates = new ArrayList<CharSequence>();

        // Check AllArtifactNameCompleter
        final Completer allArtifactNameCompleter = new AllArtifactNameCompleter(this.petalsAdminApi
                .getPetalsAdministrationFactory().newArtifactAdministration());
        allArtifactNameCompleter.complete("", 0, candidates);
        assertNotNull(candidates);
        assertEquals("AllArtifactNameCompleter: Unexpected candidates", new HashSet<CharSequence>(allArtifacts),
                new HashSet<CharSequence>(candidates));

        // Check StartedArtifactNameCompleter
        candidates.clear();
        final Completer startedArtifactNameCompleter = new StartedArtifactNameCompleter(this.petalsAdminApi
                .getPetalsAdministrationFactory().newArtifactAdministration());
        startedArtifactNameCompleter.complete("", 0, candidates);
        assertNotNull(candidates);
        assertEquals("StartedArtifactNameCompleter: Unexpected candidates", new HashSet<CharSequence>(
                artifactsThatCanBeStopped), new HashSet<CharSequence>(candidates));

        // Check StoppedArtifactNameCompleter
        candidates.clear();
        final Completer stoppedArtifactNameCompleter = new StoppedArtifactNameCompleter(this.petalsAdminApi
                .getPetalsAdministrationFactory().newArtifactAdministration());
        stoppedArtifactNameCompleter.complete("", 0, candidates);
        assertNotNull(candidates);
        assertEquals("StoppedArtifactNameCompleter: Unexpected candidates", new HashSet<CharSequence>(
                artifactsThatCanBeStarted), new HashSet<CharSequence>(candidates));
    }

}
